@@ -31,7 +31,7 @@
 #include <rpm/rpmtypes.h>
 #include <rpm/rpmlog.h>
 #include <rpm/rpmmacro.h>
-#include <rpm/rpmfileutil.h>	/* rpmioMkpath */
+#include <rpm/rpmfileutil.h>
 #include <rpm/rpmstring.h>
 #include <rpm/rpmdb.h>
 
@@ -101,17 +101,18 @@ static int sqlInRoot = 0;
 static void enterChroot(dbiIndex dbi)
 {
     int xx;
+    rpmdb db = dbi->dbi_rpmdb;
 
-    if ((dbi->dbi_root[0] == '/' && dbi->dbi_root[1] == '\0') || dbi->dbi_rpmdb->db_chrootDone || sqlInRoot)
+    if (rstreq(db->db_root, "/") || db->db_chrootDone || sqlInRoot)
        /* Nothing to do, was not already in chroot */
        return;
 
 if (_debug)
-fprintf(stderr, "sql:chroot(%s)\n", dbi->dbi_root);
+fprintf(stderr, "sql:chroot(%s)\n", db->db_root);
 
     sqlCwd = rpmGetCwd();
     xx = chdir("/");
-    xx = chroot(dbi->dbi_root);
+    xx = chroot(db->db_root);
 assert(xx == 0);
     sqlInRoot=1;
 }
@@ -119,8 +120,9 @@ assert(xx == 0);
 static void leaveChroot(dbiIndex dbi)
 {
     int xx;
+    rpmdb db = dbi->dbi_rpmdb;
 
-    if ((dbi->dbi_root[0] == '/' && dbi->dbi_root[1] == '\0') || dbi->dbi_rpmdb->db_chrootDone || !sqlInRoot)
+    if (rstreq(db->db_root, "/") || db->db_chrootDone || !sqlInRoot)
        /* Nothing to do, not in chroot */
        return;
 
@@ -336,7 +338,7 @@ fprintf(stderr, "sqlite3_step: DONE scp %p [%d:%d] av %p avlen %p\n", scp, scp->
 		vtype = sqlite3_column_decltype(scp->pStmt, i);
 		nb = 0;
 
-		if (!strcmp(vtype, "blob")) {
+		if (rstreq(vtype, "blob")) {
 		    const void * v = sqlite3_column_blob(scp->pStmt, i);
 		    nb = sqlite3_column_bytes(scp->pStmt, i);
 if (_debug)
@@ -348,7 +350,7 @@ fprintf(stderr, "\t%d %s %s %p[%zd]\n", i, cname, vtype, v, nb);
 			scp->ac++;
 		    }
 		} else
-		if (!strcmp(vtype, "double")) {
+		if (rstreq(vtype, "double")) {
 		    double v = sqlite3_column_double(scp->pStmt, i);
 		    nb = sizeof(v);
 if (_debug)
@@ -360,7 +362,7 @@ assert(dbiByteSwapped(dbi) == 0); /* Byte swap?! */
 			scp->ac++;
 		    }
 		} else
-		if (!strcmp(vtype, "int")) {
+		if (rstreq(vtype, "int")) {
 		    int32_t v = sqlite3_column_int(scp->pStmt, i);
 		    nb = sizeof(v);
 if (_debug)
@@ -379,7 +381,7 @@ if (dbiByteSwapped(dbi) == 1)
 			scp->ac++;
 		    }
 		} else
-		if (!strcmp(vtype, "int64")) {
+		if (rstreq(vtype, "int64")) {
 		    int64_t v = sqlite3_column_int64(scp->pStmt, i);
 		    nb = sizeof(v);
 if (_debug)
@@ -391,7 +393,7 @@ assert(dbiByteSwapped(dbi) == 0); /* Byte swap?! */
 			scp->ac++;
 		    }
 		} else
-		if (!strcmp(vtype, "text")) {
+		if (rstreq(vtype, "text")) {
 		    const char * v = (const char *) sqlite3_column_text(scp->pStmt, i);
 		    nb = strlen(v) + 1;
 if (_debug)
@@ -770,14 +772,10 @@ enterChroot(dbi);
 	rpmlog(RPMLOG_DEBUG, "closed   sql db         %s\n",
 		dbi->dbi_subfile);
 
-	dbi->dbi_stats = _free(dbi->dbi_stats);
-	dbi->dbi_file = _free(dbi->dbi_file);
-	dbi->dbi_db = _free(dbi->dbi_db);
-
 leaveChroot(dbi);
     }
 
-    dbi = _free(dbi);
+    dbi = db3Free(dbi);
 
     return rc;
 }
@@ -793,15 +791,11 @@ static int sql_open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
 {
     extern const struct _dbiVec sqlitevec;
    
-    char * root;
-    char * home;
     char * dbhome;
-    const char * dbfile;  
     char * dbfname;
     const char * sql_errcode;
     dbiIndex dbi;
     SQL_DB * sqldb;
-    size_t len;
     int rc = 0;
     int xx;
     
@@ -814,42 +808,15 @@ static int sql_open(rpmdb rpmdb, rpmTag rpmtag, dbiIndex * dbip)
     if ((dbi = db3New(rpmdb, rpmtag)) == NULL)
 	return 1;
 
-   /*
-     * Get the prefix/root component and directory path
-     */
-    root = rpmdb->db_root;
-    home = rpmdb->db_home;
-    
-    dbi->dbi_root = root;
-    dbi->dbi_home = home;
-      
-    dbfile = rpmTagGetName(dbi->dbi_rpmtag);
-
 enterChroot(dbi);
 
-    /*
-     * Make a copy of the tagName result..
-     * use this for the filename and table name
-     */
-    {	
-      char * t;
-      len = strlen(dbfile);
-      t = xcalloc(len + 1, sizeof(*t));
-      (void) stpcpy( t, dbfile );
-      dbi->dbi_file = t;
-/* WRONG */
-      dbi->dbi_subfile = t;
-    }
+    /* sqlite uses subfile for table name */
+    dbi->dbi_subfile = xstrdup(dbi->dbi_file);
 
     dbi->dbi_mode=O_RDWR;
        
-    dbhome = rpmGenPath(NULL, home, NULL);
+    dbhome = rpmGenPath(NULL, rpmdb->db_home, NULL);
 
-    /* 
-     * Create the /var/lib/rpm directory if it doesn't exist (root only).
-     */
-    (void) rpmioMkpath(dbhome, 0755, getuid(), getgid());
-       
     dbfname = rpmGenPath(dbhome, dbi->dbi_file, NULL);
 
     rpmlog(RPMLOG_DEBUG, "opening  sql db         %s (%s) mode=0x%x\n",
