@@ -79,9 +79,10 @@ static const size_t headerMaxbytes = (32*1024*1024);
  *
  * @param h		header
  * @retval td		tag data container
+ * @param flags		modifier flags
  * @return		0 on success
  */
-typedef int (*headerTagTagFunction) (Header h, rpmtd td);
+typedef int (*headerTagTagFunction) (Header h, rpmtd td, headerGetFlags hgflags);
 
 extern void *rpmHeaderTagFunc(rpmTag tag);
 
@@ -225,7 +226,6 @@ unsigned headerSizeof(Header h, enum hMagic magicp)
 {
     indexEntry entry;
     unsigned int size = 0;
-    unsigned int pad = 0;
     int i;
 
     if (h == NULL)
@@ -244,7 +244,6 @@ unsigned headerSizeof(Header h, enum hMagic magicp)
     size += 2 * sizeof(int32_t);	/* count of index entries */
 
     for (i = 0, entry = h->index; i < h->indexUsed; i++, entry++) {
-	unsigned diff;
 	rpmTagType type;
 
 	/* Regions go in as is ... */
@@ -263,10 +262,9 @@ unsigned headerSizeof(Header h, enum hMagic magicp)
 	/* Alignment */
 	type = entry->info.type;
 	if (typeSizes[type] > 1) {
-	    diff = typeSizes[type] - (size % typeSizes[type]);
+	    unsigned diff = typeSizes[type] - (size % typeSizes[type]);
 	    if (diff != typeSizes[type]) {
 		size += diff;
-		pad += diff;
 	    }
 	}
 
@@ -342,19 +340,8 @@ static int dataLength(rpmTagType type, rpm_constdata_t p, rpm_count_t count,
 /** \ingroup header
  * Swap int32_t and int16_t arrays within header region.
  *
- * This code is way more twisty than I would like.
- *
- * A bug with RPM_I18NSTRING_TYPE in rpm-2.5.x (fixed in August 1998)
- * causes the offset and length of elements in a header region to disagree
- * regarding the total length of the region data.
- *
- * The "fix" is to compute the size using both offset and length and
- * return the larger of the two numbers as the size of the region.
- * Kinda like computing left and right Riemann sums of the data elements
- * to determine the size of a data structure, go figger :-).
- *
- * There's one other twist if a header region tag is in the set to be swabbed,
- * as the data for a header region is located after all other tag data.
+ * If a header region tag is in the set to be swabbed, as the data for a
+ * a header region is located after all other tag data.
  *
  * @param entry		header entry
  * @param il		no. of entries
@@ -371,13 +358,6 @@ static int regionSwab(indexEntry entry, int il, int dl,
 		const unsigned char * dataEnd,
 		int regionid)
 {
-    unsigned char * tprev = NULL;
-    unsigned char * t = NULL;
-    int tdel = 0;
-    int tl = dl;
-    struct indexEntry_s ieprev;
-
-    memset(&ieprev, 0, sizeof(ieprev));
     for (; il > 0; il--, pe++) {
 	struct indexEntry_s ie;
 	rpmTagType type;
@@ -396,8 +376,8 @@ static int regionSwab(indexEntry entry, int il, int dl,
 	if (hdrchkAlign(ie.info.type, ie.info.offset))
 	    return -1;
 
-	ie.data = t = dataStart + ie.info.offset;
-	if (dataEnd && t >= dataEnd)
+	ie.data = dataStart + ie.info.offset;
+	if (dataEnd && (unsigned char *)ie.data >= dataEnd)
 	    return -1;
 
 	ie.length = dataLength(ie.info.type, ie.data, ie.info.count, 1, dataEnd);
@@ -415,76 +395,42 @@ static int regionSwab(indexEntry entry, int il, int dl,
 	/* Alignment */
 	type = ie.info.type;
 	if (typeSizes[type] > 1) {
-	    unsigned diff;
-	    diff = typeSizes[type] - (dl % typeSizes[type]);
+	    unsigned diff = typeSizes[type] - (dl % typeSizes[type]);
 	    if (diff != typeSizes[type]) {
 		dl += diff;
-		if (ieprev.info.type == RPM_I18NSTRING_TYPE)
-		    ieprev.length += diff;
 	    }
 	}
-	tdel = (tprev ? (t - tprev) : 0);
-	if (ieprev.info.type == RPM_I18NSTRING_TYPE)
-	    tdel = ieprev.length;
-
-	if (ie.info.tag >= HEADER_I18NTABLE) {
-	    tprev = t;
-	} else {
-	    tprev = dataStart;
-	    /* XXX HEADER_IMAGE tags don't include region sub-tag. */
-	    if (ie.info.tag == HEADER_IMAGE)
-		tprev -= REGION_TAG_COUNT;
-	}
 
 	/* Perform endian conversions */
 	switch (ntohl(pe->type)) {
 	case RPM_INT64_TYPE:
-	{   uint64_t * it = (uint64_t *)t;
+	{   uint64_t * it = ie.data;
 	    for (; ie.info.count > 0; ie.info.count--, it += 1) {
 		if (dataEnd && ((unsigned char *)it) >= dataEnd)
 		    return -1;
 		*it = htonll(*it);
 	    }
-	    t = (unsigned char *) it;
 	}   break;
 	case RPM_INT32_TYPE:
-	{   int32_t * it = (int32_t *)t;
+	{   int32_t * it = ie.data;
 	    for (; ie.info.count > 0; ie.info.count--, it += 1) {
 		if (dataEnd && ((unsigned char *)it) >= dataEnd)
 		    return -1;
 		*it = htonl(*it);
 	    }
-	    t = (unsigned char *) it;
 	}   break;
 	case RPM_INT16_TYPE:
-	{   int16_t * it = (int16_t *) t;
+	{   int16_t * it = ie.data;
 	    for (; ie.info.count > 0; ie.info.count--, it += 1) {
 		if (dataEnd && ((unsigned char *)it) >= dataEnd)
 		    return -1;
 		*it = htons(*it);
 	    }
-	    t = (unsigned char *) it;
 	}   break;
-	default:
-	    t += ie.length;
-	    break;
 	}
 
 	dl += ie.length;
-	tl += tdel;
-	ieprev = ie;	/* structure assignment */
-
     }
-    tdel = (tprev ? (t - tprev) : 0);
-    tl += tdel;
-
-    /* XXX
-     * There are two hacks here:
-     *	1) tl is 16b (i.e. REGION_TAG_COUNT) short while doing headerReload().
-     *	2) the 8/98 rpm bug with inserting i18n tags needs to use tl, not dl.
-     */
-    if (tl+REGION_TAG_COUNT == dl)
-	tl += REGION_TAG_COUNT;
 
     return dl;
 }
@@ -502,7 +448,6 @@ static void * doHeaderUnload(Header h,
     entryInfo pe;
     char * dataStart;
     char * te;
-    unsigned pad;
     unsigned len;
     int32_t il = 0;
     int32_t dl = 0;
@@ -510,15 +455,14 @@ static void * doHeaderUnload(Header h,
     rpmTagType type;
     int i;
     int drlen, ndribbles;
-    int driplen, ndrips;
-    int legacy = 0;
+
+    if (h == NULL) return NULL;
 
     /* Sort entries by (offset,tag). */
     headerUnsort(h);
 
     /* Compute (il,dl) for all tags, including those deleted in region. */
-    pad = 0;
-    drlen = ndribbles = driplen = ndrips = 0;
+    drlen = ndribbles = 0;
     for (i = 0, entry = h->index; i < h->indexUsed; i++, entry++) {
 	if (ENTRY_IS_REGION(entry)) {
 	    int32_t rdl = -entry->info.offset;	/* negative offset */
@@ -539,11 +483,9 @@ static void * doHeaderUnload(Header h,
 		/* Alignment */
 		type = entry->info.type;
 		if (typeSizes[type] > 1) {
-		    unsigned diff;
-		    diff = typeSizes[type] - (dl % typeSizes[type]);
+		    unsigned diff = typeSizes[type] - (dl % typeSizes[type]);
 		    if (diff != typeSizes[type]) {
 			drlen += diff;
-			pad += diff;
 			dl += diff;
 		    }
 		}
@@ -565,19 +507,13 @@ static void * doHeaderUnload(Header h,
 	/* Alignment */
 	type = entry->info.type;
 	if (typeSizes[type] > 1) {
-	    unsigned diff;
-	    diff = typeSizes[type] - (dl % typeSizes[type]);
+	    unsigned diff = typeSizes[type] - (dl % typeSizes[type]);
 	    if (diff != typeSizes[type]) {
-		driplen += diff;
-		pad += diff;
 		dl += diff;
-	    } else
-		diff = 0;
+	    }
 	}
 
-	ndrips++;
 	il++;
-	driplen += entry->length;
 	dl += entry->length;
     }
 
@@ -594,7 +530,6 @@ static void * doHeaderUnload(Header h,
     pe = (entryInfo) &ei[2];
     dataStart = te = (char *) (pe + il);
 
-    pad = 0;
     for (i = 0, entry = h->index; i < h->indexUsed; i++, entry++) {
 	const char * src;
 	unsigned char *t;
@@ -621,7 +556,6 @@ static void * doHeaderUnload(Header h,
 	    if (i == 0 && (h->flags & HEADERFLAG_LEGACY)) {
 		int32_t stei[4];
 
-		legacy = 1;
 		memcpy(pe+1, src, rdl);
 		memcpy(te, src + rdl, rdlen);
 		te += rdlen;
@@ -680,7 +614,6 @@ static void * doHeaderUnload(Header h,
 	    if (diff != typeSizes[type]) {
 		memset(te, 0, diff);
 		te += diff;
-		pad += diff;
 	    }
 	}
 
@@ -762,7 +695,7 @@ void * headerUnload(Header h)
 static
 indexEntry findEntry(Header h, rpmTag tag, rpmTagType type)
 {
-    indexEntry entry, entry2, last;
+    indexEntry entry;
     struct indexEntry_s key;
 
     if (h == NULL) return NULL;
@@ -770,8 +703,7 @@ indexEntry findEntry(Header h, rpmTag tag, rpmTagType type)
 
     key.info.tag = tag;
 
-    entry2 = entry = 
-	bsearch(&key, h->index, h->indexUsed, sizeof(*h->index), indexCmp);
+    entry = bsearch(&key, h->index, h->indexUsed, sizeof(*h->index), indexCmp);
     if (entry == NULL)
 	return NULL;
 
@@ -785,14 +717,6 @@ indexEntry findEntry(Header h, rpmTag tag, rpmTagType type)
     if (entry->info.tag == tag && entry->info.type == type)
 	return entry;
 
-    last = h->index + h->indexUsed;
-    /* FIX: entry2 = entry. Code looks bogus as well. */
-    while (entry2->info.tag == tag && entry2->info.type != type &&
-	   entry2 < last) entry2++;
-
-    if (entry->info.tag == tag && entry->info.type == type)
-	return entry;
-
     return NULL;
 }
 
@@ -847,7 +771,6 @@ Header headerLoad(void * uh)
     unsigned char * dataEnd;
     indexEntry entry; 
     int rdlen;
-    int i;
 
     /* Sanity checks on header intro. */
     if (hdrchkTags(il) || hdrchkData(dl))
@@ -869,7 +792,6 @@ Header headerLoad(void * uh)
     h = headerLink(h);
 
     entry = h->index;
-    i = 0;
     if (!(htonl(pe->tag) < HEADER_I18NTABLE)) {
 	h->flags |= HEADERFLAG_LEGACY;
 	entry->info.type = REGION_TAG_TYPE;
@@ -880,12 +802,9 @@ Header headerLoad(void * uh)
 	entry->data = pe;
 	entry->length = pvlen - sizeof(il) - sizeof(dl);
 	rdlen = regionSwab(entry+1, il, 0, pe, dataStart, dataEnd, entry->info.offset);
-#if 0	/* XXX don't check, the 8/98 i18n bug fails here. */
 	if (rdlen != dl)
 	    goto errxit;
-#endif
 	entry->rdlen = rdlen;
-	entry++;
 	h->indexUsed++;
     } else {
 	int32_t rdl;
@@ -966,6 +885,7 @@ Header headerLoad(void * uh)
 
     h->flags &= ~HEADERFLAG_SORTED;
     headerSort(h);
+    h->flags |= HEADERFLAG_ALLOCATED;
 
     return h;
 
@@ -991,9 +911,6 @@ Header headerReload(Header h, rpmTag tag)
 	uh = _free(uh);
 	return NULL;
     }
-    if (nh->flags & HEADERFLAG_ALLOCATED)
-	uh = _free(uh);
-    nh->flags |= HEADERFLAG_ALLOCATED;
     if (ENTRY_IS_REGION(nh->index)) {
 	if (tag == HEADER_SIGNATURES || tag == HEADER_IMMUTABLE)
 	    nh->index[0].info.tag = tag;
@@ -1014,11 +931,9 @@ Header headerCopyLoad(const void * uh)
     /* Sanity checks on header intro. */
     if (!(hdrchkTags(il) || hdrchkData(dl)) && pvlen < headerMaxbytes) {
 	nuh = memcpy(xmalloc(pvlen), uh, pvlen);
-	if ((h = headerLoad(nuh)) != NULL)
-	    h->flags |= HEADERFLAG_ALLOCATED;
+	if ((h = headerLoad(nuh)) == NULL)
+	    nuh = _free(nuh);
     }
-    if (h == NULL)
-	nuh = _free(nuh);
     return h;
 }
 
@@ -1079,12 +994,9 @@ Header headerRead(FD_t fd, enum hMagic magicp)
     h = headerLoad(ei);
 
 exit:
-    if (h) {
-	if (h->flags & HEADERFLAG_ALLOCATED)
-	    ei = _free(ei);
-	h->flags |= HEADERFLAG_ALLOCATED;
-    } else if (ei)
-	ei = _free(ei);
+    if (h == NULL && ei != NULL) {
+	free(ei);
+    }
     return h;
 }
 
@@ -1094,8 +1006,6 @@ int headerWrite(FD_t fd, Header h, enum hMagic magicp)
     size_t length;
     void * uh;
 
-    if (h == NULL)
-	return 1;
     uh = doHeaderUnload(h, &length);
     if (uh == NULL)
 	return 1;
@@ -1280,25 +1190,25 @@ static int headerMatchLocale(const char *td, const char *l, const char *le)
     const char *fe;
 
     /* First try a complete match. */
-    if (strlen(td) == (le-l) && !strncmp(td, l, (le - l)))
+    if (strlen(td) == (le-l) && rstreqn(td, l, (le - l)))
 	return 1;
 
     /* Next, try stripping optional dialect and matching.  */
     for (fe = l; fe < le && *fe != '@'; fe++)
 	{};
-    if (fe < le && !strncmp(td, l, (fe - l)))
+    if (fe < le && rstreqn(td, l, (fe - l)))
 	return 1;
 
     /* Next, try stripping optional codeset and matching.  */
     for (fe = l; fe < le && *fe != '.'; fe++)
 	{};
-    if (fe < le && !strncmp(td, l, (fe - l)))
+    if (fe < le && rstreqn(td, l, (fe - l)))
 	return 1;
 
     /* Finally, try stripping optional country code and matching. */
     for (fe = l; fe < le && *fe != '_'; fe++)
 	{};
-    if (fe < le && !strncmp(td, l, (fe - l)))
+    if (fe < le && rstreqn(td, l, (fe - l)))
 	return 2;
 
     return 0;
@@ -1376,23 +1286,18 @@ exit:
 /**
  * Retrieve tag data from header.
  * @param h		header
- * @param tag		tag to retrieve
  * @retval td		tag data container
  * @param flags		flags to control retrieval
  * @return		1 on success, 0 on not found
  */
-static int intGetTdEntry(Header h, rpmTag tag, rpmtd td, headerGetFlags flags)
+static int intGetTdEntry(Header h, rpmtd td, headerGetFlags flags)
 {
     indexEntry entry;
     int rc;
 
-    assert(td != NULL);
-    /* ensure clean state */
-    rpmtdReset(td);
-    td->tag = tag;
     /* First find the tag */
     /* FIX: h modified by sort. */
-    entry = findEntry(h, tag, RPM_NULL_TYPE);
+    entry = findEntry(h, td->tag, RPM_NULL_TYPE);
     if (entry == NULL) {
 	/* Td is zeroed above, just return... */
 	return 0;
@@ -1415,35 +1320,21 @@ static int intGetTdEntry(Header h, rpmTag tag, rpmtd td, headerGetFlags flags)
     return ((rc == 1) ? 1 : 0);
 }
 
-/* 
- * XXX temporary kludgery until tag extensions have been converted to 
- * take rpmtd as argument
- */
-static int intGetTagExt(Header h, rpmTag tag, rpmtd td, headerTagTagFunction tagfunc)
-{
-    int rc;
-    rpmtdReset(td);
-    rc = tagfunc(h, td);
-    td->tag = tag;
-    return rc; 
-}
-
 int headerGet(Header h, rpmTag tag, rpmtd td, headerGetFlags flags)
 {
     int rc;
-    headerTagTagFunction tagfunc = NULL;
+    headerTagTagFunction tagfunc = intGetTdEntry;
 
-    assert(td != NULL);
+    if (td == NULL) return 0;
+
+    rpmtdReset(td);
+    td->tag = tag;
 
     if (flags & HEADERGET_EXT) {
-	tagfunc = rpmHeaderTagFunc(tag);
-    }
-	
-    if (tagfunc) {
-	rc = intGetTagExt(h, tag, td, tagfunc);
-    } else {
-	rc = intGetTdEntry(h, tag, td, flags);
+	headerTagTagFunction extfunc = rpmHeaderTagFunc(tag);
+	if (extfunc) tagfunc = extfunc;
     }
+    rc = tagfunc(h, td, flags);
 
     assert(tag == td->tag);
     return rc;
@@ -1732,7 +1623,7 @@ int headerAddI18NString(Header h, rpmTag tag, const char * string,
 
     {	const char * l = table->data;
 	for (langNum = 0; langNum < table->info.count; langNum++) {
-	    if (!strcmp(l, lang)) break;
+	    if (rstreq(l, lang)) break;
 	    l += strlen(l) + 1;
 	}
     }
@@ -1893,15 +1784,11 @@ HeaderIterator headerInitIterator(Header h)
     return hi;
 }
 
-int headerNext(HeaderIterator hi, rpmtd td)
+static indexEntry nextIndex(HeaderIterator hi)
 {
     Header h = hi->h;
-    int slot = hi->next_index;
+    int slot;
     indexEntry entry = NULL;
-    int rc;
-
-    assert(td != NULL);
-    rpmtdReset(td);
 
     for (slot = hi->next_index; slot < h->indexUsed; slot++) {
 	entry = h->index + slot;
@@ -1910,16 +1797,28 @@ int headerNext(HeaderIterator hi, rpmtd td)
     }
     hi->next_index = slot;
     if (entry == NULL || slot >= h->indexUsed)
-	return 0;
+	return NULL;
 
-   	/* LCL: no clue */
     hi->next_index++;
+    return entry;
+}
 
-    td->tag = entry->info.tag;
+rpmTag headerNextTag(HeaderIterator hi)
+{
+    indexEntry entry = nextIndex(hi);
+    return entry ? entry->info.tag : RPMTAG_NOT_FOUND;
+}
 
-    rc = copyTdEntry(entry, td, HEADERGET_DEFAULT);
+int headerNext(HeaderIterator hi, rpmtd td)
+{
+    indexEntry entry = nextIndex(hi);
+    int rc = 0;
 
-    /* XXX 1 on success */
+    rpmtdReset(td);
+    if (entry) {
+	td->tag = entry->info.tag;
+	rc = copyTdEntry(entry, td, HEADERGET_DEFAULT);
+    }
     return ((rc == 1) ? 1 : 0);
 }
 
@@ -1958,7 +1857,7 @@ void headerCopyTags(Header headerFrom, Header headerTo,
     for (p = tagstocopy; *p != 0; p++) {
 	if (headerIsEntry(headerTo, *p))
 	    continue;
-	if (!headerGet(headerFrom, *p, &td, HEADERGET_MINMEM))
+	if (!headerGet(headerFrom, *p, &td, (HEADERGET_MINMEM|HEADERGET_RAW)))
 	    continue;
 	(void) headerPut(headerTo, &td, HEADERPUT_DEFAULT);
 	rpmtdFreeData(&td);
@@ -1974,3 +1873,45 @@ void headerSetInstance(Header h, unsigned int instance)
 {
     h->instance = instance;
 }    
+
+char * headerGetAsString(Header h, rpmTag tag)
+{
+    char *res = NULL;
+    struct rpmtd_s td;
+
+    if (headerGet(h, tag, &td, HEADERGET_EXT)) {
+	if (rpmtdCount(&td) == 1) {
+	    res = rpmtdFormat(&td, RPMTD_FORMAT_STRING, NULL);
+	}
+	rpmtdFreeData(&td);
+    }
+    return res;
+}
+
+const char * headerGetString(Header h, rpmTag tag)
+{
+    const char *res = NULL;
+    struct rpmtd_s td;
+
+    if (headerGet(h, tag, &td, HEADERGET_MINMEM)) {
+	if (rpmtdCount(&td) == 1) {
+	    res = rpmtdGetString(&td);
+	}
+	rpmtdFreeData(&td);
+    }
+    return res;
+}
+
+uint64_t headerGetNumber(Header h, rpmTag tag)
+{
+    uint64_t res = 0;
+    struct rpmtd_s td;
+
+    if (headerGet(h, tag, &td, HEADERGET_EXT)) {
+	if (rpmtdCount(&td) == 1) {
+	    res = rpmtdGetNumber(&td);
+	}
+	rpmtdFreeData(&td);
+    }
+    return res;
+}
