@@ -85,6 +85,7 @@ typedef struct headerSprintfArgs_s {
     size_t alloced;
     int numTokens;
     int i;
+    headerGetFlags hgflags;
 } * headerSprintfArgs;
 
 
@@ -157,6 +158,8 @@ static void hsaInit(headerSprintfArgs hsa)
     hsa->i = 0;
     if (tag != NULL && tag->tag == -2)
 	hsa->hi = headerInitIterator(hsa->h);
+    /* Normally with bells and whistles enabled, but raw dump on iteration. */
+    hsa->hgflags = (hsa->hi == NULL) ? HEADERGET_EXT : HEADERGET_RAW;
 }
 
 /**
@@ -235,12 +238,12 @@ static int findTag(headerSprintfArgs hsa, sprintfToken token, const char * name)
     stag->fmt = NULL;
     stag->tag = -1;
 
-    if (!strcmp(tagname, "*")) {
+    if (rstreq(tagname, "*")) {
 	stag->tag = -2;
 	goto bingo;
     }
 
-    if (strncmp("RPMTAG_", tagname, sizeof("RPMTAG_")-1) == 0) {
+    if (rstreqn("RPMTAG_", tagname, sizeof("RPMTAG_")-1)) {
 	tagname += sizeof("RPMTAG");
     }
 
@@ -289,7 +292,6 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
     sprintfToken format;
     sprintfToken token;
     int numTokens;
-    int i;
     int done = 0;
 
     /* upper limit on number of individual formats */
@@ -302,7 +304,6 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
     format = xcalloc(numTokens, sizeof(*format));
     if (endPtr) *endPtr = NULL;
 
-    /* LCL: can't detect done termination */
     dst = start = str;
     numTokens = 0;
     token = NULL;
@@ -330,10 +331,8 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 		char * newEnd;
 
 		start++;
-		if (parseExpression(hsa, token, start, &newEnd))
-		{
-		    format = freeFormat(format, numTokens);
-		    return 1;
+		if (parseExpression(hsa, token, start, &newEnd)) {
+		    goto errxit;
 		}
 		start = newEnd;
 		break;
@@ -346,8 +345,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    while (*chptr && *chptr != '{' && *chptr != '%') chptr++;
 	    if (!*chptr || *chptr == '%') {
 		hsa->errmsg = _("missing { after %");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 
 	    *chptr++ = '\0';
@@ -369,8 +367,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    while (*next && *next != '}') next++;
 	    if (!*next) {
 		hsa->errmsg = _("missing } after %{");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 	    *next++ = '\0';
 
@@ -381,8 +378,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 		*chptr++ = '\0';
 		if (!*chptr) {
 		    hsa->errmsg = _("empty tag format");
-		    format = freeFormat(format, numTokens);
-		    return 1;
+		    goto errxit;
 		}
 		token->u.tag.type = chptr;
 	    } 
@@ -393,17 +389,14 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    
 	    if (!*start) {
 		hsa->errmsg = _("empty tag name");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 
-	    i = 0;
 	    token->type = PTOK_TAG;
 
 	    if (findTag(hsa, token, start)) {
 		hsa->errmsg = _("unknown tag");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 
 	    start = next;
@@ -417,16 +410,13 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	    if (parseFormat(hsa, start,
 			    &token->u.array.format,
 			    &token->u.array.numTokens,
-			    &start, PARSER_IN_ARRAY))
-	    {
-		format = freeFormat(format, numTokens);
-		return 1;
+			    &start, PARSER_IN_ARRAY)) {
+		goto errxit;
 	    }
 
 	    if (!start) {
 		hsa->errmsg = _("] expected at end of array");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 
 	    dst = start;
@@ -438,8 +428,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	case ']':
 	    if (state != PARSER_IN_ARRAY) {
 		hsa->errmsg = _("unexpected ]");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 	    *start++ = '\0';
 	    if (endPtr) *endPtr = start;
@@ -449,8 +438,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 	case '}':
 	    if (state != PARSER_IN_EXPR) {
 		hsa->errmsg = _("unexpected }");
-		format = freeFormat(format, numTokens);
-		return 1;
+		goto errxit;
 	    }
 	    *start++ = '\0';
 	    if (endPtr) *endPtr = start;
@@ -479,7 +467,7 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
     if (dst != NULL)
         *dst = '\0';
 
-    for (i = 0; i < numTokens; i++) {
+    for (int i = 0; i < numTokens; i++) {
 	token = format + i;
 	if (token->type == PTOK_STRING)
 	    token->u.string.len = strlen(token->u.string.string);
@@ -487,8 +475,11 @@ static int parseFormat(headerSprintfArgs hsa, char * str,
 
     *numTokensPtr = numTokens;
     *formatPtr = format;
-
     return 0;
+
+errxit:
+    freeFormat(format, numTokens);
+    return 1;
 }
 
 static int parseExpression(headerSprintfArgs hsa, sprintfToken token,
@@ -614,7 +605,7 @@ static rpmtd getData(headerSprintfArgs hsa, rpmTag tag)
 
     if (!(td = getCached(hsa->cache, tag))) {
 	td = rpmtdNew();
-	if (!headerGet(hsa->h, tag, td, HEADERGET_EXT)) {
+	if (!headerGet(hsa->h, tag, td, hsa->hgflags)) {
 	    rpmtdFree(td);
 	    return NULL;
 	}
@@ -701,7 +692,7 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 	break;
 
     case PTOK_COND:
-	if (getCached(hsa->cache, token->u.cond.tag.tag) ||
+	if (getData(hsa, token->u.cond.tag.tag) ||
 		      headerIsEntry(hsa->h, token->u.cond.tag.tag)) {
 	    spft = token->u.cond.ifFormat;
 	    condNumFormats = token->u.cond.numIfTokens;
@@ -761,7 +752,7 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 
 	    spft = token->u.array.format;
 	    isxml = (spft->type == PTOK_TAG && spft->u.tag.type != NULL &&
-		!strcmp(spft->u.tag.type, "xml"));
+		    rstreq(spft->u.tag.type, "xml"));
 
 	    if (isxml) {
 		const char * tagN = rpmTagGetName(spft->u.tag.tag);
@@ -854,7 +845,7 @@ char * headerFormat(Header h, const char * fmt, errmsg_t * errmsg)
 	(hsa.format->type == PTOK_ARRAY
 	    ? &hsa.format->u.array.format->u.tag :
 	NULL));
-    isxml = (tag != NULL && tag->tag == -2 && tag->type != NULL && !strcmp(tag->type, "xml"));
+    isxml = (tag != NULL && tag->tag == -2 && tag->type != NULL && rstreq(tag->type, "xml"));
 
     if (isxml) {
 	need = sizeof("<rpmHeader>\n") - 1;
