@@ -29,13 +29,12 @@
 
 #include "lib/rpmdb_internal.h"
 #include "lib/fprint.h"
-#include "lib/header_internal.h"	/* XXX for HEADERFLAG_ALLOCATED */
+#include "lib/header_internal.h"	/* XXX for headerSetInstance() */
 #include "debug.h"
 
 int _rpmdb_debug = 0;
 
 static int _rebuildinprogress = 0;
-static int _db_filter_dups = 0;
 
 #define	_DBI_FLAGS	0
 #define	_DBI_PERMS	0644
@@ -213,6 +212,13 @@ dbiIndex dbiOpen(rpmdb db, rpmTag rpmtag, unsigned int flags)
     if ((dbi = db->_dbi[dbix]) != NULL)
 	return dbi;
 
+    /* Try to ensure db home exists, error out if we cant even create */
+    if (!db->db_mkdirDone) {
+	const char *dbhome = rpmdbHome(db);
+	db->db_mkdirDone = (rpmioMkpath(dbhome, 0755, getuid(), getgid()) == 0);
+	if (!db->db_mkdirDone) return NULL;
+    }
+
     _dbapi_rebuild = rpmExpandNumeric("%{_dbapi_rebuild}");
     if (_dbapi_rebuild < 1 || _dbapi_rebuild > 4)
 	_dbapi_rebuild = 4;
@@ -714,6 +720,7 @@ static int unblockSignals(sigset_t * oldMask)
 
 #define	_DB_ROOT	"/"
 #define	_DB_HOME	"%{_dbpath}"
+#define _DB_FULLPATH	NULL
 #define	_DB_FLAGS	0
 #define _DB_MODE	0
 #define _DB_PERMS	0644
@@ -722,7 +729,7 @@ static int unblockSignals(sigset_t * oldMask)
 #define	_DB_ERRPFX	"rpmdb"
 
 static struct rpmdb_s const dbTemplate = {
-    _DB_ROOT,	_DB_HOME, _DB_FLAGS, _DB_MODE, _DB_PERMS,
+    _DB_ROOT,	_DB_HOME, _DB_FULLPATH, _DB_FLAGS, _DB_MODE, _DB_PERMS,
     _DB_MAJOR,	_DB_ERRPFX
 };
 
@@ -761,6 +768,15 @@ rpmop rpmdbOp(rpmdb rpmdb, rpmdbOpX opx)
     return op;
 }
 
+const char *rpmdbHome(rpmdb db)
+{
+    const char *dbdir = NULL;
+    if (db) {
+	dbdir = db->db_chrootDone ? db->db_home : db->db_fullpath;
+    }
+    return dbdir;
+}
+
 int rpmdbSetChrootDone(rpmdb db, int chrootDone)
 {
     int ochrootDone = 0;
@@ -840,6 +856,7 @@ int rpmdbClose(rpmdb db)
     db->db_errpfx = _free(db->db_errpfx);
     db->db_root = _free(db->db_root);
     db->db_home = _free(db->db_home);
+    db->db_fullpath = _free(db->db_fullpath);
     db->db_bits = PBM_FREE(db->db_bits);
     db->_dbi = _free(db->_dbi);
 
@@ -893,7 +910,6 @@ rpmdb newRpmdb(const char * root,
     static int _initialized = 0;
 
     if (!_initialized) {
-	_db_filter_dups = rpmExpandNumeric("%{_filterdbdups}");
 	_initialized = 1;
     }
 
@@ -907,24 +923,22 @@ rpmdb newRpmdb(const char * root,
     if (perms >= 0)	db->db_perms = perms;
     if (flags >= 0)	db->db_flags = flags;
 
-    if (root && *root) {
-	db->db_root = rpmGetPath(root, NULL);
-    } else
-	db->db_root = rpmGetPath(_DB_ROOT, NULL);
-
     db->db_home = rpmGetPath( (home && *home ? home : _DB_HOME), NULL);
     if (!(db->db_home && db->db_home[0] != '%')) {
 	rpmlog(RPMLOG_ERR, _("no dbpath has been set\n"));
-	db->db_root = _free(db->db_root);
 	db->db_home = _free(db->db_home);
 	db = _free(db);
 	return NULL;
     }
+    db->db_root = rpmGetPath((root && *root) ? root : _DB_ROOT, NULL);
+    db->db_fullpath = rpmGenPath(db->db_root, db->db_home, NULL);
     db->db_errpfx = rpmExpand( (epfx && *epfx ? epfx : _DB_ERRPFX), NULL);
     /* XXX remove environment after chrooted operations, for now... */
-    db->db_remove_env = ((strcmp(db->db_root, "/") != 0)? 1 : 0);
-    db->db_filter_dups = _db_filter_dups;
+    db->db_remove_env = (!rstreq(db->db_root, "/") ? 1 : 0);
     db->db_ndbi = dbiTags.max;
+    db->db_malloc = rmalloc;
+    db->db_realloc = rrealloc;
+    db->db_free = NULL; /* XXX rfree() prototype differs from free() */
     db->_dbi = xcalloc(db->db_ndbi, sizeof(*db->_dbi));
     db->nrefs = 0;
     return rpmdbLink(db, RPMDBG_M("rpmdbCreate"));
@@ -1126,9 +1140,6 @@ static int rpmdbFindByFile(rpmdb db, const char * filespec,
     if (baseName == NULL)
 	goto exit;
 
-    fpc = fpCacheCreate(20);
-    fp1 = fpLookup(fpc, dirName, baseName, 1);
-
     dbi = dbiOpen(db, RPMTAG_BASENAMES, 0);
     if (dbi != NULL) {
 	dbcursor = NULL;
@@ -1154,12 +1165,14 @@ static int rpmdbFindByFile(rpmdb db, const char * filespec,
     } else
 	rc = -2;
 
-    if (rc) goto exit;
+    if (rc || allMatches == NULL) goto exit;
 
     *matches = xcalloc(1, sizeof(**matches));
     rec = dbiIndexNewItem(0, 0);
+    fpc = fpCacheCreate(allMatches->count);
+    fp1 = fpLookup(fpc, dirName, baseName, 1);
+
     i = 0;
-    if (allMatches != NULL)
     while (i < allMatches->count) {
 	struct rpmtd_s bn, dn, di;
 	const char ** baseNames, ** dirNames;
@@ -1212,6 +1225,7 @@ static int rpmdbFindByFile(rpmdb db, const char * filespec,
     }
 
     rec = _free(rec);
+    fpCacheFree(fpc);
 
     if ((*matches)->count == 0) {
 	*matches = dbiFreeIndexSet(*matches);
@@ -1222,7 +1236,6 @@ static int rpmdbFindByFile(rpmdb db, const char * filespec,
 
 exit:
     dbiFreeIndexSet(allMatches);
-    fpCacheFree(fpc);
     free(dirName);
     return rc;
 }
@@ -1564,7 +1577,6 @@ rpmdbMatchIterator rpmdbFreeIterator(rpmdbMatchIterator mi)
 	mire->pattern = _free(mire->pattern);
 	if (mire->preg != NULL) {
 	    regfree(mire->preg);
-	    /* LCL: regfree has bogus only */
 	    mire->preg = _free(mire->preg);
 	}
     }
@@ -1605,8 +1617,7 @@ static int miregexec(miRE mire, const char * val)
 
     switch (mire->mode) {
     case RPMMIRE_STRCMP:
-	rc = strcmp(mire->pattern, val);
-	if (rc) rc = 1;
+	rc = (!rstreq(mire->pattern, val));
 	break;
     case RPMMIRE_DEFAULT:
     case RPMMIRE_REGEX:
@@ -1757,13 +1768,13 @@ int rpmdbSetIteratorRE(rpmdbMatchIterator mi, rpmTag tag,
     if (defmode == (rpmMireMode)-1) {
 	char *t = rpmExpand("%{?_query_selector_match}", NULL);
 
-	if (*t == '\0' || !strcmp(t, "default"))
+	if (*t == '\0' || rstreq(t, "default"))
 	    defmode = RPMMIRE_DEFAULT;
-	else if (!strcmp(t, "strcmp"))
+	else if (rstreq(t, "strcmp"))
 	    defmode = RPMMIRE_STRCMP;
-	else if (!strcmp(t, "regex"))
+	else if (rstreq(t, "regex"))
 	    defmode = RPMMIRE_REGEX;
-	else if (!strcmp(t, "glob"))
+	else if (rstreq(t, "glob"))
 	    defmode = RPMMIRE_GLOB;
 	else
 	    defmode = RPMMIRE_DEFAULT;
@@ -1812,7 +1823,6 @@ int rpmdbSetIteratorRE(rpmdbMatchIterator mi, rpmTag tag,
 	allpat = _free(allpat);
 	if (preg) {
 	    regfree(preg);
-	    /* LCL: regfree has bogus only */
 	    preg = _free(preg);
 	}
 	return rc;
@@ -2028,8 +2038,10 @@ top:
     } while (mi->mi_offset == 0);
 
     /* If next header is identical, return it now. */
-    if (mi->mi_prevoffset && mi->mi_offset == mi->mi_prevoffset)
-	return mi->mi_h;
+    if (mi->mi_prevoffset && mi->mi_offset == mi->mi_prevoffset) {
+	/* ...but rpmdb record numbers are unique, avoid endless loop */
+	return (mi->mi_rpmtag == RPMDBI_PACKAGES) ? NULL : mi->mi_h;
+    }
 
     /* Retrieve next header blob for index iterator. */
     if (uh == NULL) {
@@ -2099,8 +2111,6 @@ top:
     /* Did the header blob load correctly? */
 #if !defined(_USE_COPY_LOAD)
     mi->mi_h = headerLoad(uh);
-    if (mi->mi_h)
-	mi->mi_h->flags |= HEADERFLAG_ALLOCATED;
 #else
     mi->mi_h = headerCopyLoad(uh);
 #endif
@@ -2151,7 +2161,6 @@ void rpmdbSortIterator(rpmdbMatchIterator mi)
     }
 }
 
-/* LCL: segfault */
 static int rpmdbGrowIterator(rpmdbMatchIterator mi)
 {
     DBC * dbcursor;
@@ -2524,7 +2533,7 @@ int rpmdbRemove(rpmdb db, int rid, unsigned int hdrNum,
     }
 
     {	
-	char *nevra = headerGetNEVRA(h, NULL);
+	char *nevra = headerGetAsString(h, RPMTAG_NEVRA);
 	rpmlog(RPMLOG_DEBUG, "  --- h#%8u %s\n", hdrNum, nevra);
 	free(nevra);
     }
@@ -2884,7 +2893,7 @@ int rpmdbAdd(rpmdb db, int iid, Header h,
 			const char **tnames = tagdata.data;
 			const char *str = rpmtdGetString(&tagdata);
 			for (j = 0; j < i; j++) {
-			    if (!strcmp(str, tnames[j]))
+			    if (rstreq(str, tnames[j]))
 				break;
 			}
 			if (j < i)
@@ -3148,7 +3157,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     tfn = _free(tfn);
 
     tfn = rpmGetPath("%{?_dbpath_rebuild}", NULL);
-    if (!(tfn && tfn[0] != '\0' && strcmp(tfn, dbpath)))
+    if (!(tfn && tfn[0] != '\0' && !rstreq(tfn, dbpath)))
     {
 	tfn = _free(tfn);
 	rasprintf(&tfn, "%srebuilddb.%d", dbpath, (int) getpid());
@@ -3223,30 +3232,6 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 		continue;
 	    }
 
-	    /* Filter duplicate entries ? (bug in pre rpm-3.0.4) */
-	    if (_db_filter_dups || newdb->db_filter_dups) {
-		const char * name, * version, * release;
-		int skip = 0;
-
-		(void) headerNVR(h, &name, &version, &release);
-
-		{   rpmdbMatchIterator mi;
-		    mi = rpmdbInitIterator(newdb, RPMTAG_NAME, name, 0);
-		    (void) rpmdbSetIteratorRE(mi, RPMTAG_VERSION,
-				RPMMIRE_DEFAULT, version);
-		    (void) rpmdbSetIteratorRE(mi, RPMTAG_RELEASE,
-				RPMMIRE_DEFAULT, release);
-		    while (rpmdbNextIterator(mi)) {
-			skip = 1;
-			break;
-		    }
-		    mi = rpmdbFreeIterator(mi);
-		}
-
-		if (skip)
-		    continue;
-	    }
-
 	    /* Deleted entries are eliminated in legacy headers by copy. */
 	    {	Header nh = (headerIsEntry(h, RPMTAG_HEADERIMAGE)
 				? headerCopy(h) : NULL);
