@@ -147,7 +147,7 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 	else {
 	    const char * flink = rpmfiFLink(fi);
 	    linkto[size] = '\0';
-	    if (flink == NULL || strcmp(linkto, flink))
+	    if (flink == NULL || !rstreq(linkto, flink))
 		*res |= RPMVERIFY_LINKTO;
 	}
     } 
@@ -241,14 +241,14 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
     if (flags & RPMVERIFY_USER) {
 	const char * name = uidToUname(sb.st_uid);
 	const char * fuser = rpmfiFUser(fi);
-	if (name == NULL || fuser == NULL || strcmp(name, fuser))
+	if (name == NULL || fuser == NULL || !rstreq(name, fuser))
 	    *res |= RPMVERIFY_USER;
     }
 
     if (flags & RPMVERIFY_GROUP) {
 	const char * name = gidToGname(sb.st_gid);
 	const char * fgroup = rpmfiFGroup(fi);
-	if (name == NULL || fgroup == NULL || strcmp(name, fgroup))
+	if (name == NULL || fgroup == NULL || !rstreq(name, fgroup))
 	    *res |= RPMVERIFY_GROUP;
     }
 
@@ -308,11 +308,10 @@ static int verifyHeader(QVA_t qva, const rpmts ts, Header h)
     rpmVerifyAttrs omitMask = ((qva->qva_flags & VERIFY_ATTRS) ^ VERIFY_ATTRS);
     int ec = 0;		/* assume no problems */
     char *buf = NULL;
-    int i;
 
     rpmfi fi = rpmfiNew(ts, h, RPMTAG_BASENAMES, RPMFI_FLAGS_VERIFY);
     rpmfiInit(fi, 0);
-    while ((i = rpmfiNext(fi)) >= 0) {
+    while (rpmfiNext(fi) >= 0) {
 	rpmfileAttrs fileAttrs;
 	int rc;
 
@@ -349,7 +348,7 @@ static int verifyHeader(QVA_t qva, const rpmts ts, Header h)
 	    static const char *const aok = ".";
 	    static const char *const unknown = "?";
 
-	    ec = (verifyResult != 0);
+	    if (verifyResult) ec = 1;
 
 #define	_verify(_RPMVERIFY_F, _C)	\
 	((verifyResult & _RPMVERIFY_F) ? _C : aok)
@@ -418,7 +417,7 @@ static int verifyDependencies(QVA_t qva, rpmts ts,
 
     psi = rpmpsInitIterator(ps);
     if (rpmpsNumProblems(ps) > 0) {
-	char *nevra = headerGetNEVRA(h, NULL);
+	char *nevra = headerGetAsString(h, RPMTAG_NEVRA);
 	rpmlog(RPMLOG_NOTICE, _("Unsatisfied dependencies for %s:\n"), nevra);
 	free(nevra);
 	while (rpmpsNextIterator(psi) >= 0) {
@@ -443,12 +442,8 @@ int showVerifyPackage(QVA_t qva, rpmts ts, Header h)
     int rc;
 
     if (qva->qva_flags & VERIFY_DEPS) {
-	int save_noise = _rpmds_unspecified_epoch_noise;
-	if (rpmIsVerbose())
-	    _rpmds_unspecified_epoch_noise = 1;
 	if ((rc = verifyDependencies(qva, ts, h)) != 0)
 	    ec = rc;
-	_rpmds_unspecified_epoch_noise = save_noise;
     }
     if (qva->qva_flags & VERIFY_FILES) {
 	if ((rc = verifyHeader(qva, ts, h)) != 0)
@@ -471,8 +466,9 @@ int rpmcliVerify(rpmts ts, QVA_t qva, char * const * argv)
 {
     rpmVSFlags vsflags, ovsflags;
     int ec = 0, xx;
+    int dirfd = -1;
     const char * rootDir = rpmtsRootDir(ts);
-    int cachingDeps = _cacheDependsRC;
+    FD_t scriptFd = fdDup(STDOUT_FILENO);
 
     /* 
      * Open the DB + indices explicitly before possible chroot,
@@ -480,15 +476,14 @@ int rpmcliVerify(rpmts ts, QVA_t qva, char * const * argv)
      */
     rpmtsOpenDB(ts, O_RDONLY);
     rpmdbOpenAll(rpmtsGetRdb(ts));
-    if (rootDir && strcmp(rootDir, "/") != 0) {
-	if (chroot(rootDir) == -1) {
+    if (rootDir && !rstreq(rootDir, "/")) {
+	dirfd = open(".", O_RDONLY);
+	if (dirfd == -1 || chdir("/") == -1 || chroot(rootDir) == -1) {
 	    rpmlog(RPMLOG_ERR, _("Unable to change root directory: %m\n"));
 	    ec = 1;
 	    goto exit;
 	} else {
 	    rpmtsSetChrootDone(ts, 1);
-	    /* XXX temporary db path is wrong when chrooted, disable caching */
-	    _cacheDependsRC = 0;
 	}
     }
 
@@ -517,11 +512,13 @@ int rpmcliVerify(rpmts ts, QVA_t qva, char * const * argv)
     if (rpmtsChrootDone(ts)) {
 	/* only done if previous chroot succeeded, assume success */
 	xx = chroot(".");
+	xx = fchdir(dirfd);
 	rpmtsSetChrootDone(ts, 0);
     }
 
 exit:
-    _cacheDependsRC = cachingDeps;
+    Fclose(scriptFd);
+    if (dirfd >= 0) close(dirfd);
 
     return ec;
 }
