@@ -1,8 +1,4 @@
-/** \ingroup py_c
- * \file python/rpmmodule.c
- */
-
-#include "system.h"
+#include "rpmsystem-py.h"
 
 #include <rpm/rpmlib.h>		/* rpmMachineScore, rpmReadConfigFiles */
 #include <rpm/rpmtag.h>
@@ -11,16 +7,16 @@
 #include <rpm/rpmlog.h>
 
 #include "header-py.h"
-#include "rpmal-py.h"
 #include "rpmds-py.h"
 #include "rpmfd-py.h"
 #include "rpmfi-py.h"
+#include "rpmkeyring-py.h"
 #include "rpmmi-py.h"
 #include "rpmps-py.h"
 #include "rpmmacro-py.h"
+#include "rpmtd-py.h"
 #include "rpmte-py.h"
 #include "rpmts-py.h"
-#include "spec-py.h"
 
 #include "debug.h"
 
@@ -28,147 +24,98 @@
  * \name Module: rpm
  */
 
-/**
- */
 PyObject * pyrpmError;
 
-/**
- */
-static PyObject * archScore(PyObject * self, PyObject * args, PyObject * kwds)
+static PyObject * archScore(PyObject * self, PyObject * arg)
 {
     char * arch;
-    int score;
-    char * kwlist[] = {"arch", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s", kwlist, &arch))
+    if (!PyArg_Parse(arg, "s", &arch))
 	return NULL;
 
-    score = rpmMachineScore(RPM_MACHTABLE_INSTARCH, arch);
-
-    return Py_BuildValue("i", score);
+    return Py_BuildValue("i", rpmMachineScore(RPM_MACHTABLE_INSTARCH, arch));
 }
 
-/**
- *  */
-static PyObject * signalsCaught(PyObject * self, PyObject * check)
+static PyObject * signalCaught(PyObject *self, PyObject *o)
 {
-    PyObject *caught, *o;
-    int llen;
-    int signum, i;
-    sigset_t newMask, oldMask;
+    int signo;
+    if (!PyArg_Parse(o, "i", &signo)) return NULL;
 
-    if (!PyList_Check(check)) {
-	PyErr_SetString(PyExc_TypeError, "list expected");
-	return NULL;
-    }
-
-    llen = PyList_Size(check);
-    caught = PyList_New(0);
-
-    /* block signals while checking for them */
-    (void) sigfillset(&newMask);
-    (void) sigprocmask(SIG_BLOCK, &newMask, &oldMask);
-
-    for (i = 0; i < llen; i++) {
-	o = PyList_GetItem(check, i);
-	signum = PyInt_AsLong(o);
-	if (rpmsqIsCaught(signum) > 0) {
-	    PyList_Append(caught, o);
-	}
-    }
-    (void) sigprocmask(SIG_SETMASK, &oldMask, NULL);
-
-    return caught;
+    return PyBool_FromLong(rpmsqIsCaught(signo));
 }
 
-/**
- *  */
-static PyObject * checkSignals(PyObject * self, PyObject * args)
+static PyObject * checkSignals(PyObject * self)
 {
-    if (!PyArg_ParseTuple(args, ":checkSignals")) return NULL;
     rpmdbCheckSignals();
-    Py_INCREF(Py_None);
-    return Py_None;
+    Py_RETURN_NONE;
 }
 
-
-/**
- */
-static PyObject * setLogFile (PyObject * self, PyObject * args, PyObject *kwds)
+static PyObject * setLogFile (PyObject * self, PyObject *arg)
 {
-    PyObject * fop = NULL;
-    FILE * fp = NULL;
-    char * kwlist[] = {"fileObject", NULL};
+    FILE *fp;
+    int fdno = PyObject_AsFileDescriptor(arg);
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "|O:logSetFile", kwlist, &fop))
-	return NULL;
-
-    if (fop) {
-	if (!PyFile_Check(fop)) {
-	    PyErr_SetString(pyrpmError, "requires file object");
+    if (fdno >= 0) {
+	/* XXX we dont know the mode here.. guessing append for now */
+	fp = fdopen(fdno, "a");
+	if (fp == NULL) {
+	    PyErr_SetFromErrno(PyExc_IOError);
 	    return NULL;
 	}
-	fp = PyFile_AsFile(fop);
+    } else if (arg == Py_None) {
+	fp = NULL;
+    } else {
+	PyErr_SetString(PyExc_TypeError, "file object or None expected");
+	return NULL;
     }
 
     (void) rpmlogSetFile(fp);
-
-    Py_INCREF(Py_None);
-    return (PyObject *) Py_None;
+    Py_RETURN_NONE;
 }
 
-/**
- */
 static PyObject *
-setVerbosity (PyObject * self, PyObject * args, PyObject *kwds)
+setVerbosity (PyObject * self, PyObject * arg)
 {
     int level;
-    char * kwlist[] = {"level", NULL};
 
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &level))
+    if (!PyArg_Parse(arg, "i", &level))
 	return NULL;
 
     rpmSetVerbosity(level);
 
-    Py_INCREF(Py_None);
-    return (PyObject *) Py_None;
+    Py_RETURN_NONE;
 }
 
-/**
- */
 static PyObject *
-setEpochPromote (PyObject * self, PyObject * args, PyObject * kwds)
+setEpochPromote (PyObject * self, PyObject * arg)
 {
-    char * kwlist[] = {"promote", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist,
-	    &_rpmds_nopromote))
+    if (!PyArg_Parse(arg, "i", &_rpmds_nopromote))
 	return NULL;
 
-    Py_INCREF(Py_None);
-    return (PyObject *) Py_None;
+    Py_RETURN_NONE;
 }
 
-/**
- */
-static PyObject * setStats (PyObject * self, PyObject * args, PyObject * kwds)
+static PyObject * setStats (PyObject * self, PyObject * arg)
 {
-    char * kwlist[] = {"stats", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &_rpmts_stats))
+    if (!PyArg_Parse(arg, "i", &_rpmts_stats))
 	return NULL;
 
-    Py_INCREF(Py_None);
-    return (PyObject *) Py_None;
+    Py_RETURN_NONE;
+}
+
+static PyObject * doLog(PyObject * self, PyObject * args, PyObject *kwds)
+{
+    int code;
+    const char *msg;
+    char * kwlist[] = {"code", "msg", NULL};
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "is", kwlist, &code, &msg))
+	return NULL;
+
+    rpmlog(code, "%s", msg);
+    Py_RETURN_NONE;
 }
 
-/**
- */
 static PyMethodDef rpmModuleMethods[] = {
-    { "TransactionSet", (PyCFunction) rpmts_Create, METH_VARARGS|METH_KEYWORDS,
-"rpm.TransactionSet([rootDir, [db]]) -> ts\n\
-- Create a transaction set.\n" },
-
     { "addMacro", (PyCFunction) rpmmacro_AddMacro, METH_VARARGS|METH_KEYWORDS,
 	NULL },
     { "delMacro", (PyCFunction) rpmmacro_DelMacro, METH_VARARGS|METH_KEYWORDS,
@@ -176,42 +123,33 @@ static PyMethodDef rpmModuleMethods[] = {
     { "expandMacro", (PyCFunction) rpmmacro_ExpandMacro, METH_VARARGS|METH_KEYWORDS,
 	NULL },
 
-    { "archscore", (PyCFunction) archScore, METH_VARARGS|METH_KEYWORDS,
+    { "archscore", (PyCFunction) archScore, METH_O,
 	NULL },
 
-    { "signalsCaught", (PyCFunction) signalsCaught, METH_O, 
+    { "signalCaught", (PyCFunction) signalCaught, METH_O, 
 	NULL },
-    { "checkSignals", (PyCFunction) checkSignals, METH_VARARGS,
+    { "checkSignals", (PyCFunction) checkSignals, METH_NOARGS,
         NULL },
 
-    { "headerLoad", (PyCFunction) hdrLoad, METH_VARARGS|METH_KEYWORDS,
-	NULL },
     { "mergeHeaderListFromFD", (PyCFunction) rpmMergeHeadersFromFD, METH_VARARGS|METH_KEYWORDS,
 	NULL },
-    { "readHeaderListFromFD", (PyCFunction) rpmHeaderFromFD, METH_VARARGS|METH_KEYWORDS,
-	NULL },
-    { "readHeaderListFromFile", (PyCFunction) rpmHeaderFromFile, METH_VARARGS|METH_KEYWORDS,
-	NULL },
-    { "readHeaderFromFD", (PyCFunction) rpmSingleHeaderFromFD, METH_VARARGS|METH_KEYWORDS,
-	NULL },
 
-    { "setLogFile", (PyCFunction) setLogFile, METH_VARARGS|METH_KEYWORDS,
+    { "log",		(PyCFunction) doLog, METH_VARARGS|METH_KEYWORDS,
+	NULL },
+    { "setLogFile", (PyCFunction) setLogFile, METH_O,
 	NULL },
 
     { "versionCompare", (PyCFunction) versionCompare, METH_VARARGS|METH_KEYWORDS,
 	NULL },
     { "labelCompare", (PyCFunction) labelCompare, METH_VARARGS|METH_KEYWORDS,
 	NULL },
-    { "setVerbosity", (PyCFunction) setVerbosity, METH_VARARGS|METH_KEYWORDS,
+    { "setVerbosity", (PyCFunction) setVerbosity, METH_O,
 	NULL },
-    { "setEpochPromote", (PyCFunction) setEpochPromote, METH_VARARGS|METH_KEYWORDS,
+    { "setEpochPromote", (PyCFunction) setEpochPromote, METH_O,
 	NULL },
-    { "setStats", (PyCFunction) setStats, METH_VARARGS|METH_KEYWORDS,
+    { "setStats", (PyCFunction) setStats, METH_O,
 	NULL },
 
-    { "dsSingle", (PyCFunction) rpmds_Single, METH_VARARGS|METH_KEYWORDS,
-"rpm.dsSingle(TagN, N, [EVR, [Flags]] -> ds\n\
-- Create a single element dependency set.\n" },
     { NULL }
 } ;
 
@@ -223,64 +161,131 @@ static void rpm_exithook(void)
    rpmdbCheckTerminate(1);
 }
 
-/**
- */
 static char rpm__doc__[] =
 "";
 
-void init_rpm(void);	/* XXX eliminate gcc warning */
-/**
+/*
+ * Add rpm tag dictionaries to the module
  */
+static void addRpmTags(PyObject *module)
+{
+    PyObject *pyval, *pyname, *dict = PyDict_New();
+    rpmtd names = rpmtdNew();
+    rpmTagGetNames(names, 1);
+    const char *tagname, *shortname;
+    rpmTag tagval;
+
+    while ((tagname = rpmtdNextString(names))) {
+	shortname = tagname + strlen("RPMTAG_");
+	tagval = rpmTagGetValue(shortname);
+
+	PyModule_AddIntConstant(module, tagname, tagval);
+	pyval = PyInt_FromLong(tagval);
+	pyname = Py_BuildValue("s", shortname);
+	PyDict_SetItem(dict, pyval, pyname);
+	Py_DECREF(pyval);
+	Py_DECREF(pyname);
+    }
+    PyModule_AddObject(module, "tagnames", dict);
+    rpmtdFreeData(names);
+    rpmtdFree(names);
+}
+
+/*
+  Do any common preliminary work before python 2 vs python 3 module creation:
+*/
+static int prepareInitModule(void)
+{
+    if (PyType_Ready(&hdr_Type) < 0) return 0;
+    if (PyType_Ready(&rpmds_Type) < 0) return 0;
+    if (PyType_Ready(&rpmfd_Type) < 0) return 0;
+    if (PyType_Ready(&rpmfi_Type) < 0) return 0;
+    if (PyType_Ready(&rpmKeyring_Type) < 0) return 0;
+    if (PyType_Ready(&rpmmi_Type) < 0) return 0;
+    if (PyType_Ready(&rpmProblem_Type) < 0) return 0;
+    if (PyType_Ready(&rpmPubkey_Type) < 0) return 0;
+#if 0
+    if (PyType_Ready(&rpmtd_Type) < 0) return 0;
+#endif
+    if (PyType_Ready(&rpmte_Type) < 0) return 0;
+    if (PyType_Ready(&rpmts_Type) < 0) return 0;
+
+    return 1;
+}
+static int initModule(PyObject *m);
+
+#if PY_MAJOR_VERSION >= 3
+static int rpmModuleTraverse(PyObject *m, visitproc visit, void *arg) {
+    Py_VISIT(pyrpmError);
+    return 0;
+}
+
+static int rpmModuleClear(PyObject *m) {
+    Py_CLEAR(pyrpmError);
+    return 0;
+}
+
+static struct PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT,
+    "_rpm",            /* m_name */
+    rpm__doc__,        /* m_doc */
+    0,                 /* m_size */
+    rpmModuleMethods,
+    NULL,              /* m_reload */
+    rpmModuleTraverse,
+    rpmModuleClear,
+    NULL               /* m_free */
+};
+
+PyObject *
+PyInit__rpm(void);
+
+PyObject *
+PyInit__rpm(void)
+{
+    PyObject * m;
+    if (!prepareInitModule()) return NULL;
+    m = PyModule_Create(&moduledef);
+    initModule(m);
+    return m;
+}
+#else
+void init_rpm(void);	/* XXX eliminate gcc warning */
 void init_rpm(void)
 {
-    PyObject * d, *o, *tag = NULL, *dict, *m;
-
-#if Py_TPFLAGS_HAVE_ITER        /* XXX backport to python-1.5.2 */
-    if (PyType_Ready(&hdr_Type) < 0) return;
-    if (PyType_Ready(&rpmal_Type) < 0) return;
-    if (PyType_Ready(&rpmds_Type) < 0) return;
-    if (PyType_Ready(&rpmfd_Type) < 0) return;
-    if (PyType_Ready(&rpmfi_Type) < 0) return;
-    if (PyType_Ready(&rpmmi_Type) < 0) return;
-    if (PyType_Ready(&rpmps_Type) < 0) return;
-
-    if (PyType_Ready(&rpmte_Type) < 0) return;
-    if (PyType_Ready(&rpmts_Type) < 0) return;
-    if (PyType_Ready(&spec_Type) < 0) return;
-#endif
+    PyObject * m;
 
+    if (!prepareInitModule()) return;
     m = Py_InitModule3("_rpm", rpmModuleMethods, rpm__doc__);
     if (m == NULL)
 	return;
+    initModule(m);
+}
+#endif
+
+/* Shared python2/3 module initialization: */
+static int initModule(PyObject *m)
+{
+    PyObject * d;
 
     /* 
      * treat error to register rpm cleanup hook as fatal, tracebacks
      * can and will leave stale locks around if we can't clean up
      */
     if (Py_AtExit(rpm_exithook) == -1)
-	return;
+        return 0;
 
     rpmReadConfigFiles(NULL, NULL);
 
     d = PyModule_GetDict(m);
 
-#ifdef	HACK
-    pyrpmError = PyString_FromString("_rpm.error");
-    PyDict_SetItemString(d, "error", pyrpmError);
-    Py_DECREF(pyrpmError);
-#else
     pyrpmError = PyErr_NewException("_rpm.error", NULL, NULL);
     if (pyrpmError != NULL)
 	PyDict_SetItemString(d, "error", pyrpmError);
-#endif
 
-#if Py_TPFLAGS_HAVE_ITER        /* XXX backport to python-1.5.2 */
     Py_INCREF(&hdr_Type);
     PyModule_AddObject(m, "hdr", (PyObject *) &hdr_Type);
 
-    Py_INCREF(&rpmal_Type);
-    PyModule_AddObject(m, "al", (PyObject *) &rpmal_Type);
-
     Py_INCREF(&rpmds_Type);
     PyModule_AddObject(m, "ds", (PyObject *) &rpmds_Type);
 
@@ -290,11 +295,22 @@ void init_rpm(void)
     Py_INCREF(&rpmfi_Type);
     PyModule_AddObject(m, "fi", (PyObject *) &rpmfi_Type);
 
+    Py_INCREF(&rpmKeyring_Type);
+    PyModule_AddObject(m, "keyring", (PyObject *) &rpmKeyring_Type);
+
     Py_INCREF(&rpmmi_Type);
     PyModule_AddObject(m, "mi", (PyObject *) &rpmmi_Type);
 
-    Py_INCREF(&rpmps_Type);
-    PyModule_AddObject(m, "ps", (PyObject *) &rpmps_Type);
+    Py_INCREF(&rpmProblem_Type);
+    PyModule_AddObject(m, "prob", (PyObject *) &rpmProblem_Type);
+
+    Py_INCREF(&rpmPubkey_Type);
+    PyModule_AddObject(m, "pubkey", (PyObject *) &rpmPubkey_Type);
+
+#if 0
+    Py_INCREF(&rpmtd_Type);
+    PyModule_AddObject(m, "td", (PyObject *) &rpmtd_Type);
+#endif
 
     Py_INCREF(&rpmte_Type);
     PyModule_AddObject(m, "te", (PyObject *) &rpmte_Type);
@@ -302,48 +318,20 @@ void init_rpm(void)
     Py_INCREF(&rpmts_Type);
     PyModule_AddObject(m, "ts", (PyObject *) &rpmts_Type);
 
-    Py_INCREF(&spec_Type);
-    PyModule_AddObject(m, "spec", (PyObject *) &spec_Type);
-#else
-    hdr_Type.ob_type = &PyType_Type;
-    rpmal_Type.ob_type = &PyType_Type;
-    rpmds_Type.ob_type = &PyType_Type;
-    rpmfd_Type.ob_type = &PyType_Type;
-    rpmfi_Type.ob_type = &PyType_Type;
-    rpmmi_Type.ob_type = &PyType_Type;
-    rpmps_Type.ob_type = &PyType_Type;
-    rpmte_Type.ob_type = &PyType_Type;
-    rpmts_Type.ob_type = &PyType_Type;
-    spec_Type.ob_type =  &PyType_Type;
-#endif
+    addRpmTags(m);
 
-    dict = PyDict_New();
-    {	const char *tname, *sname;
-	rpmtd names = rpmtdNew();
-	rpmTagGetNames(names, 1);
+    PyModule_AddStringConstant(m, "__version__", RPMVERSION);
 
-	while ((tname = rpmtdNextString(names))) {
-	    sname = tname + strlen("RPMTAG_");
-	    tag = PyInt_FromLong(rpmTagGetValue(sname));
-	    PyDict_SetItemString(d, tname, tag);
-	    Py_DECREF(tag);
-	    o = PyString_FromString(sname);
-	    PyDict_SetItem(dict, tag, o);
-	    Py_DECREF(o);
-	}
-	rpmtdFreeData(names);
-	rpmtdFree(names);
-    }	
-    PyDict_SetItemString(d, "tagnames", dict);
-    Py_DECREF(dict);
-
-
-#define REGISTER_ENUM(val) \
-    PyDict_SetItemString(d, #val, o=PyInt_FromLong( val )); \
-    Py_DECREF(o);
+#define REGISTER_ENUM(val) PyModule_AddIntConstant(m, #val, val)
 
     REGISTER_ENUM(RPMTAG_NOT_FOUND);
 
+    REGISTER_ENUM(RPMRC_OK);
+    REGISTER_ENUM(RPMRC_NOTFOUND);
+    REGISTER_ENUM(RPMRC_FAIL);
+    REGISTER_ENUM(RPMRC_NOTTRUSTED);
+    REGISTER_ENUM(RPMRC_NOKEY);
+
     REGISTER_ENUM(RPMFILE_STATE_NORMAL);
     REGISTER_ENUM(RPMFILE_STATE_REPLACED);
     REGISTER_ENUM(RPMFILE_STATE_NOTINSTALLED);
@@ -352,7 +340,6 @@ void init_rpm(void)
 
     REGISTER_ENUM(RPMFILE_CONFIG);
     REGISTER_ENUM(RPMFILE_DOC);
-    REGISTER_ENUM(RPMFILE_ICON);
     REGISTER_ENUM(RPMFILE_MISSINGOK);
     REGISTER_ENUM(RPMFILE_NOREPLACE);
     REGISTER_ENUM(RPMFILE_GHOST);
@@ -365,6 +352,7 @@ void init_rpm(void)
     REGISTER_ENUM(RPMDEP_SENSE_REQUIRES);
     REGISTER_ENUM(RPMDEP_SENSE_CONFLICTS);
 
+    REGISTER_ENUM(RPMSENSE_ANY);
     REGISTER_ENUM(RPMSENSE_LESS);
     REGISTER_ENUM(RPMSENSE_GREATER);
     REGISTER_ENUM(RPMSENSE_EQUAL);
@@ -398,6 +386,7 @@ void init_rpm(void)
     REGISTER_ENUM(RPMTRANS_FLAG_NODOCS);
     REGISTER_ENUM(RPMTRANS_FLAG_ALLFILES);
     REGISTER_ENUM(RPMTRANS_FLAG_KEEPOBSOLETE);
+    REGISTER_ENUM(RPMTRANS_FLAG_NOCONTEXTS);
     REGISTER_ENUM(RPMTRANS_FLAG_REPACKAGE);
     REGISTER_ENUM(RPMTRANS_FLAG_REVERSE);
     REGISTER_ENUM(RPMTRANS_FLAG_NOPRE);
@@ -491,7 +480,12 @@ void init_rpm(void)
     REGISTER_ENUM(TR_REMOVED);
 
     REGISTER_ENUM(RPMDBI_PACKAGES);
+    REGISTER_ENUM(RPMDBI_LABEL);
 
-    REGISTER_ENUM((intptr_t)RPMAL_NOMATCH);
+    REGISTER_ENUM(HEADERCONV_EXPANDFILELIST);
+    REGISTER_ENUM(HEADERCONV_COMPRESSFILELIST);
+    REGISTER_ENUM(HEADERCONV_RETROFIT_V3);
+
+    return 1;
 }
 
