@@ -1,7 +1,7 @@
 #ifndef	H_RPMPGP
 #define	H_RPMPGP
 
-/** \ingroup rpmio
+/** \ingroup rpmpgp
  * \file rpmio/rpmpgp.h
  *
  * OpenPGP constants and structures from RFC-2440.
@@ -10,30 +10,40 @@
  *	Copyright (C) The Internet Society (1998).  All Rights Reserved.
  */
 
-#include <popt.h>
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <rpm/rpmtypes.h>
+#include <rpm/rpmstring.h>
 
-#if !defined(_BEECRYPT_API_H)
-/*@-redef@*/
-typedef unsigned char byte;
-/*@=redef@*/
-#endif	/* _BEECRYPT_API_H */
+#ifdef __cplusplus
+extern "C" {
+#endif
 
-/**
+/** \ingroup rpmpgp
  */
-typedef /*@abstract@*/ struct DIGEST_CTX_s * DIGEST_CTX;
+typedef struct DIGEST_CTX_s * DIGEST_CTX;
+typedef struct rpmDigestBundle_s * rpmDigestBundle;
 
-/**
+/** \ingroup rpmpgp
  */
-typedef /*@abstract@*/ struct pgpPkt_s * pgpPkt;
+typedef struct pgpDig_s * pgpDig;
 
-/**
+/** \ingroup rpmpgp
+ */
+typedef struct pgpDigParams_s * pgpDigParams;
+
+typedef uint8_t pgpKeyID_t[8];
+typedef uint8_t pgpTime_t[4];
+
+/** \ingroup rpmpgp
  */
 typedef const struct pgpValTbl_s {
     int val;
-/*@observer@*/ const char * str;
+    char const * const str;
 } * pgpValTbl;
  
-/**
+/** \ingroup rpmpgp
  * 4.3. Packet Tags
  * 
  * The packet tag denotes what type of packet the body holds. Note that
@@ -66,12 +76,7 @@ typedef enum pgpTag_e {
     PGPTAG_CONTROL		= 63  /*!< Control (GPG) */
 } pgpTag;
 
-/**
- */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpTagTbl[];
-
-/**
+/** \ingroup rpmpgp
  * 5.1. Public-Key Encrypted Session Key Packets (Tag 1)
  *
  * A Public-Key Encrypted Session Key packet holds the session key used
@@ -106,19 +111,18 @@ extern struct pgpValTbl_s pgpTagTbl[];
  *   - MPI of Elgamal (Diffie-Hellman) value m * y**k mod p.
  */
 typedef struct pgpPktPubkey_s {
-    byte version;	/*!< version number (generate 3, accept 2). */
-    byte keyid[8];	/*!< key ID of the public key for session key. */
-    byte algo;		/*!< public key algorithm used. */
+    uint8_t version;	/*!< version number (generate 3, accept 2). */
+    pgpKeyID_t keyid;	/*!< key ID of the public key for session key. */
+    uint8_t algo;		/*!< public key algorithm used. */
 } pgpPktPubkey;
 
 
-/**
+/** \ingroup rpmpgp
  * 5.2.1. Signature Types
  * 
  * There are a number of possible meanings for a signature, which are
  * specified in a signature type octet in any given signature.
  */
-/*@-typeuse@*/
 typedef enum pgpSigType_e {
     PGPSIGTYPE_BINARY		 = 0x00, /*!< Binary document */
     PGPSIGTYPE_TEXT		 = 0x01, /*!< Canonical text document */
@@ -138,14 +142,8 @@ typedef enum pgpSigType_e {
     PGPSIGTYPE_CERT_REVOKE	 = 0x30, /*!< Certification revocation */
     PGPSIGTYPE_TIMESTAMP	 = 0x40  /*!< Timestamp */
 } pgpSigType;
-/*@=typeuse@*/
 
-/**
- */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpSigTypeTbl[];
-
-/**
+/** \ingroup rpmpgp
  * 9.1. Public Key Algorithms
  *
 \verbatim
@@ -168,7 +166,6 @@ extern struct pgpValTbl_s pgpSigTypeTbl[];
  * encryption. Implementations SHOULD implement RSA keys.
  * Implementations MAY implement any other algorithm.
  */
-/*@-typeuse@*/
 typedef enum pgpPubkeyAlgo_e {
     PGPPUBKEYALGO_RSA		=  1,	/*!< RSA */
     PGPPUBKEYALGO_RSA_ENCRYPT	=  2,	/*!< RSA(Encrypt-Only) */
@@ -180,14 +177,8 @@ typedef enum pgpPubkeyAlgo_e {
     PGPPUBKEYALGO_ELGAMAL	= 20,	/*!< Elgamal */
     PGPPUBKEYALGO_DH		= 21	/*!< Diffie-Hellman (X9.42) */
 } pgpPubkeyAlgo;
-/*@=typeuse@*/
 
-/**
- */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpPubkeyTbl[];
-
-/**
+/** \ingroup rpmpgp
  * 9.2. Symmetric Key Algorithms
  *
 \verbatim
@@ -211,7 +202,6 @@ extern struct pgpValTbl_s pgpPubkeyTbl[];
  * implement IDEA and CAST5. Implementations MAY implement any other
  * algorithm.
  */
-/*@-typeuse@*/
 typedef enum pgpSymkeyAlgo_e {
     PGPSYMKEYALGO_PLAINTEXT	=  0,	/*!< Plaintext */
     PGPSYMKEYALGO_IDEA		=  1,	/*!< IDEA */
@@ -226,15 +216,8 @@ typedef enum pgpSymkeyAlgo_e {
     PGPSYMKEYALGO_TWOFISH	= 10,	/*!< TWOFISH(256-bit key) */
     PGPSYMKEYALGO_NOENCRYPT	= 110	/*!< no encryption */
 } pgpSymkeyAlgo;
-/*@=typeuse@*/
 
-/**
- * Symmetric key (string, value) pairs.
- */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpSymkeyTbl[];
-
-/**
+/** \ingroup rpmpgp
  * 9.3. Compression Algorithms
  *
 \verbatim
@@ -249,22 +232,14 @@ extern struct pgpValTbl_s pgpSymkeyTbl[];
  * Implementations MUST implement uncompressed data. Implementations
  * SHOULD implement ZIP. Implementations MAY implement ZLIB.
  */
-/*@-typeuse@*/
 typedef enum pgpCompressAlgo_e {
     PGPCOMPRESSALGO_NONE	=  0,	/*!< Uncompressed */
     PGPCOMPRESSALGO_ZIP		=  1,	/*!< ZIP */
     PGPCOMPRESSALGO_ZLIB	=  2,	/*!< ZLIB */
     PGPCOMPRESSALGO_BZIP2	=  3	/*!< BZIP2 */
 } pgpCompressAlgo;
-/*@=typeuse@*/
 
-/**
- * Compression (string, value) pairs.
- */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpCompressionTbl[];
-
-/**
+/** \ingroup rpmpgp
  * 9.4. Hash Algorithms
  *
 \verbatim
@@ -287,36 +262,17 @@ extern struct pgpValTbl_s pgpCompressionTbl[];
  */
 typedef enum pgpHashAlgo_e {
     PGPHASHALGO_MD5		=  1,	/*!< MD5 */
-    PGPHASHALGO_SHA1		=  2,	/*!< SHA-1 */
-    PGPHASHALGO_RIPEMD160	=  3,	/*!< RIPEMD-160 */
+    PGPHASHALGO_SHA1		=  2,	/*!< SHA1 */
+    PGPHASHALGO_RIPEMD160	=  3,	/*!< RIPEMD160 */
     PGPHASHALGO_MD2		=  5,	/*!< MD2 */
-    PGPHASHALGO_TIGER192	=  6,	/*!< TIGER-192 */
+    PGPHASHALGO_TIGER192	=  6,	/*!< TIGER192 */
     PGPHASHALGO_HAVAL_5_160	=  7,	/*!< HAVAL-5-160 */
-    PGPHASHALGO_SHA256		=  8,	/*!< SHA-256 */
-    PGPHASHALGO_SHA384		=  9,	/*!< SHA-384 */
-    PGPHASHALGO_SHA512		= 10,	/*!< SHA-512 */
-
-    PGPHASHALGO_MD4		= 104,	/*!< (private) MD4 */
-    PGPHASHALGO_RIPEMD128	= 105,	/*!< (private) RIPEMD-128 */
-    PGPHASHALGO_CRC32		= 106,	/*!< (private) CRC-32 */
-    PGPHASHALGO_ADLER32		= 107,	/*!< (private) ADLER-32 */
-    PGPHASHALGO_CRC64		= 108,	/*!< (private) CRC-64 */
-    PGPHASHALGO_JLU32		= 109,	/*!< (private) Jenkins lookup3.c */
-    PGPHASHALGO_SHA224		= 110,	/*!< (private) SHA-224 */
-    PGPHASHALGO_RIPEMD256	= 111,	/*!< (private) RIPEMD-256 */
-    PGPHASHALGO_RIPEMD320	= 112,	/*!< (private) RIPEMD-320 */
-    PGPHASHALGO_SALSA10		= 113,	/*!< (private) SALSA-10 */
-    PGPHASHALGO_SALSA20		= 114,	/*!< (private) SALSA-20 */
-
+    PGPHASHALGO_SHA256		=  8,	/*!< SHA256 */
+    PGPHASHALGO_SHA384		=  9,	/*!< SHA384 */
+    PGPHASHALGO_SHA512		= 10,	/*!< SHA512 */
 } pgpHashAlgo;
 
-/**
- * Hash (string, value) pairs.
- */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpHashTbl[];
-
-/**
+/** \ingroup rpmpgp
  * 5.2.2. Version 3 Signature Packet Format
  * 
  * The body of a version 3 Signature Packet contains:
@@ -338,17 +294,17 @@ extern struct pgpValTbl_s pgpHashTbl[];
  *   - MPI of DSA value s.
  */
 typedef struct pgpPktSigV3_s {
-    byte version;	/*!< version number (3). */
-    byte hashlen;	/*!< length of following hashed material. MUST be 5. */
-    byte sigtype;	/*!< signature type. */
-    byte time[4];	/*!< 4 byte creation time. */
-    byte signid[8];	/*!< key ID of signer. */
-    byte pubkey_algo;	/*!< public key algorithm. */
-    byte hash_algo;	/*!< hash algorithm. */
-    byte signhash16[2];	/*!< left 16 bits of signed hash value. */
+    uint8_t version;	/*!< version number (3). */
+    uint8_t hashlen;	/*!< length of following hashed material. MUST be 5. */
+    uint8_t sigtype;	/*!< signature type. */
+    pgpTime_t time;	/*!< 4 byte creation time. */
+    pgpKeyID_t signid;	/*!< key ID of signer. */
+    uint8_t pubkey_algo;	/*!< public key algorithm. */
+    uint8_t hash_algo;	/*!< hash algorithm. */
+    uint8_t signhash16[2];	/*!< left 16 bits of signed hash value. */
 } * pgpPktSigV3;
 
-/**
+/** \ingroup rpmpgp
  * 5.2.3. Version 4 Signature Packet Format
  * 
  * The body of a version 4 Signature Packet contains:
@@ -370,14 +326,14 @@ typedef struct pgpPktSigV3_s {
  *   - One or more multi-precision integers comprising the signature.
  */
 typedef struct pgpPktSigV4_s {
-    byte version;	/*!< version number (4). */
-    byte sigtype;	/*!< signature type. */
-    byte pubkey_algo;	/*!< public key algorithm. */
-    byte hash_algo;	/*!< hash algorithm. */
-    byte hashlen[2];	/*!< length of following hashed material. */
+    uint8_t version;	/*!< version number (4). */
+    uint8_t sigtype;	/*!< signature type. */
+    uint8_t pubkey_algo;	/*!< public key algorithm. */
+    uint8_t hash_algo;	/*!< hash algorithm. */
+    uint8_t hashlen[2];	/*!< length of following hashed material. */
 } * pgpPktSigV4;
 
-/**
+/** \ingroup rpmpgp
  * 5.2.3.1. Signature Subpacket Specification
  * 
  * The subpacket fields consist of zero or more signature subpackets.
@@ -443,7 +399,6 @@ typedef struct pgpPktSigV4_s {
  * marked critical but is unknown to the evaluating software, the
  * evaluator SHOULD consider the signature to be in error.
  */
-/*@-typeuse@*/
 typedef enum pgpSubType_e {
     PGPSUBTYPE_NONE		=   0, /*!< none */
     PGPSUBTYPE_SIG_CREATE_TIME	=   2, /*!< signature creation time */
@@ -484,15 +439,8 @@ typedef enum pgpSubType_e {
 
     PGPSUBTYPE_CRITICAL		= 128  /*!< critical subpacket marker */
 } pgpSubType;
-/*@=typeuse@*/
 
-/**
- * Subtype (string, value) pairs.
- */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpSubTypeTbl[];
-
-/**
+/** \ingroup rpmpgp
  * 5.2. Signature Packet (Tag 2)
  *
  * A signature packet describes a binding between some public key and
@@ -546,12 +494,12 @@ typedef union pgpPktSig_u {
  *
  */
 typedef struct pgpPktSymkey_s {
-    byte version;	/*!< version number (4). */
-    byte symkey_algo;
-    byte s2k[1];
+    uint8_t version;	/*!< version number (4). */
+    uint8_t symkey_algo;
+    uint8_t s2k[1];
 } pgpPktSymkey;
 
-/**
+/** \ingroup rpmpgp
  * 5.4. One-Pass Signature Packets (Tag 4)
  *
  * The One-Pass Signature packet precedes the signed data and contains
@@ -581,15 +529,15 @@ typedef struct pgpPktSymkey_s {
  * pass packet.
  */
 typedef struct pgpPktOnepass_s {
-    byte version;	/*!< version number (3). */
-    byte sigtype;	/*!< signature type. */
-    byte hash_algo;	/*!< hash algorithm. */
-    byte pubkey_algo;	/*!< public key algorithm. */
-    byte signid[8];	/*!< key ID of signer. */
-    byte nested;
+    uint8_t version;	/*!< version number (3). */
+    uint8_t sigtype;	/*!< signature type. */
+    uint8_t hash_algo;	/*!< hash algorithm. */
+    uint8_t pubkey_algo;	/*!< public key algorithm. */
+    pgpKeyID_t signid;	/*!< key ID of signer. */
+    uint8_t nested;
 } * pgpPktOnepass;
 
-/**
+/** \ingroup rpmpgp
  * 5.5.1. Key Packet Variants
  *
  * 5.5.1.1. Public Key Packet (Tag 6)
@@ -662,13 +610,13 @@ typedef struct pgpPktOnepass_s {
  *
  */
 typedef struct pgpPktKeyV3_s {
-    byte version;	/*!< version number (3). */
-    byte time[4];	/*!< time that the key was created. */
-    byte valid[2];	/*!< time in days that this key is valid. */
-    byte pubkey_algo;	/*!< public key algorithm. */
+    uint8_t version;	/*!< version number (3). */
+    pgpTime_t time;	/*!< time that the key was created. */
+    uint8_t valid[2];	/*!< time in days that this key is valid. */
+    uint8_t pubkey_algo;	/*!< public key algorithm. */
 } * pgpPktKeyV3;
 
-/**
+/** \ingroup rpmpgp
  * The version 4 format is similar to the version 3 format except for
  * the absence of a validity period.  This has been moved to the
  * signature packet.  In addition, fingerprints of version 4 keys are
@@ -700,12 +648,12 @@ typedef struct pgpPktKeyV3_s {
  *
  */
 typedef struct pgpPktKeyV4_s {
-    byte version;	/*!< version number (4). */
-    byte time[4];	/*!< time that the key was created. */
-    byte pubkey_algo;	/*!< public key algorithm. */
+    uint8_t version;	/*!< version number (4). */
+    pgpTime_t time;	/*!< time that the key was created. */
+    uint8_t pubkey_algo;	/*!< public key algorithm. */
 } * pgpPktKeyV4;
 
-/**
+/** \ingroup rpmpgp
  * 5.5.3. Secret Key Packet Formats
  *
  * The Secret Key and Secret Subkey packets contain all the data of the
@@ -774,7 +722,7 @@ typedef union pgpPktKey_u {
     struct pgpPktKeyV4_s v4;
 } pgpPktKey;
 
-/*
+/* \ingroup rpmpgp
  * 5.6. Compressed Data Packet (Tag 8)
  *
  * The Compressed Data packet contains compressed data. Typically, this
@@ -799,11 +747,11 @@ typedef union pgpPktKey_u {
  * blocks.
  */
 typedef struct pgpPktCdata_s {
-    byte compressalgo;
-    byte data[1];
+    uint8_t compressalgo;
+    uint8_t data[1];
 } pgpPktCdata;
 
-/*
+/* \ingroup rpmpgp
  * 5.7. Symmetrically Encrypted Data Packet (Tag 9)
  *
  * The Symmetrically Encrypted Data packet contains data encrypted with
@@ -838,10 +786,10 @@ typedef struct pgpPktCdata_s {
  * session key is incorrect.
  */
 typedef struct pgpPktEdata_s {
-    byte data[1];
+    uint8_t data[1];
 } pgpPktEdata;
 
-/*
+/* \ingroup rpmpgp
  * 5.8. Marker Packet (Obsolete Literal Packet) (Tag 10)
  *
  * An experimental version of PGP used this packet as the Literal
@@ -857,7 +805,7 @@ typedef struct pgpPktEdata_s {
  * in order to cause that version to report that newer software is
  * necessary to process the message.
  */
-/*
+/* \ingroup rpmpgp
  * 5.9. Literal Data Packet (Tag 11)
  *
  * A Literal Data packet contains the body of a message; data that is
@@ -889,12 +837,12 @@ typedef struct pgpPktEdata_s {
  * the receiving software.
  */
 typedef struct pgpPktLdata_s {
-    byte format;
-    byte filenamelen;
-    byte filename[1];
+    uint8_t format;
+    uint8_t filenamelen;
+    uint8_t filename[1];
 } pgpPktLdata;
 
-/*
+/* \ingroup rpmpgp
  * 5.10. Trust Packet (Tag 12)
  *
  * The Trust packet is used only within keyrings and is not normally
@@ -908,10 +856,10 @@ typedef struct pgpPktLdata_s {
  * other than local keyring files.
  */
 typedef struct pgpPktTrust_s {
-    byte flag;
+    uint8_t flag;
 } pgpPktTrust;
 
-/*
+/* \ingroup rpmpgp
  * 5.11. User ID Packet (Tag 13)
  *
  * A User ID packet consists of data that is intended to represent the
@@ -922,10 +870,10 @@ typedef struct pgpPktTrust_s {
  *
  */
 typedef struct pgpPktUid_s {
-    byte userid[1];
+    uint8_t userid[1];
 } pgpPktUid;
 
-/**
+/** \ingroup rpmpgp
  */
 union pgpPktPre_u {
     pgpPktPubkey pubkey;	/*!< 5.1. Public-Key Encrypted Session Key */
@@ -941,9 +889,8 @@ union pgpPktPre_u {
     pgpPktUid uid;		/*!< 5.11. User ID */
 };
 
-/**
+/** \ingroup rpmpgp
  */
-/*@-typeuse@*/
 typedef enum pgpArmor_e {
     PGPARMOR_ERR_CRC_CHECK		= -7,
     PGPARMOR_ERR_BODY_DECODE		= -6,
@@ -962,17 +909,9 @@ typedef enum pgpArmor_e {
     PGPARMOR_PRIVKEY		=  6, /*!< PRIVATE KEY BLOCK */
     PGPARMOR_SECKEY		=  7  /*!< SECRET KEY BLOCK */
 } pgpArmor;
-/*@=typeuse@*/
 
-/**
- * Armor (string, value) pairs.
+/** \ingroup rpmpgp
  */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpArmorTbl[];
-
-/**
- */
-/*@-typeuse@*/
 typedef enum pgpArmorKey_e {
     PGPARMORKEY_VERSION		= 1, /*!< Version: */
     PGPARMORKEY_COMMENT		= 2, /*!< Comment: */
@@ -980,62 +919,60 @@ typedef enum pgpArmorKey_e {
     PGPARMORKEY_HASH		= 4, /*!< Hash: */
     PGPARMORKEY_CHARSET		= 5  /*!< Charset: */
 } pgpArmorKey;
-/*@=typeuse@*/
 
-/**
- * Armor key (string, value) pairs.
- */
-/*@observer@*/ /*@unchecked@*/ /*@unused@*/
-extern struct pgpValTbl_s pgpArmorKeyTbl[];
+typedef enum pgpValType_e {
+    PGPVAL_TAG			= 1,
+    PGPVAL_ARMORBLOCK		= 2,
+    PGPVAL_ARMORKEY		= 3,
+    PGPVAL_SIGTYPE		= 4,
+    PGPVAL_SUBTYPE		= 5,
+    PGPVAL_PUBKEYALGO		= 6,
+    PGPVAL_SYMKEYALGO		= 7,
+    PGPVAL_COMPRESSALGO		= 8,
+    PGPVAL_HASHALGO		= 9,
+    PGPVAL_SERVERPREFS		= 10,
+} pgpValType;
 
-/** \ingroup rpmio
+/** \ingroup rpmpgp
  * Bit(s) to control digest operation.
  */
 typedef enum rpmDigestFlags_e {
     RPMDIGEST_NONE	= 0
 } rpmDigestFlags;
 
-/*@unchecked@*/
-extern pgpHashAlgo rpmDigestHashAlgo;
+/** \ingroup rpmpgp
+ * Return string representation of am OpenPGP value.
+ * @param type		type of value
+ * @param val		byte value to lookup
+ * @return		string value of byte
+ */
+const char * pgpValString(pgpValType type, uint8_t val);
 
-/*@unchecked@*/ /*@observer@*/
-extern struct poptOption rpmDigestPoptTable[];
-
-/*@-fcnuse@*/
-#ifdef __cplusplus
-extern "C" {
-#endif
-
-/**
+/** \ingroup rpmpgp
  * Return (native-endian) integer from big-endian representation.
  * @param s		pointer to big-endian integer
  * @param nbytes	no. of bytes
  * @return		native-endian integer
  */
-/*@unused@*/ static inline
-unsigned int pgpGrab(const byte *s, int nbytes)
-	/*@*/
+static inline
+unsigned int pgpGrab(const uint8_t *s, size_t nbytes)
 {
-    unsigned int i = 0;
-    int nb = (nbytes <= sizeof(i) ? nbytes : sizeof(i));
-/*@-boundsread@*/
+    size_t i = 0;
+    size_t nb = (nbytes <= sizeof(i) ? nbytes : sizeof(i));
     while (nb--)
 	i = (i << 8) | *s++;
-/*@=boundsread@*/
     return i;
 }
 
-/**
+/** \ingroup rpmpgp
  * Return length of an OpenPGP packet.
  * @param s		pointer to packet
  * @retval *lenp	no. of bytes in packet
  * @return		no. of bytes in length prefix
  */
-/*@unused@*/ static inline
-int pgpLen(const byte *s, /*@out@*/ unsigned int *lenp)
-	/*@modifies *lenp @*/
+static inline
+size_t pgpLen(const uint8_t *s, size_t * lenp)
 {
-/*@-boundswrite@*/
     if (*s < 192) {
 	(*lenp) = *s++;
 	return 1;
@@ -1043,248 +980,40 @@ int pgpLen(const byte *s, /*@out@*/ unsigned int *lenp)
 	(*lenp) = ((((unsigned)s[0]) - 192) << 8) + s[1] + 192;
 	return 2;
     } else {
-	(*lenp) = pgpGrab(s+1, 4);
+	(*lenp) = pgpGrab(s+1, (size_t) 4);
 	return 5;
     }
-/*@=boundswrite@*/
 }
 
-/**
- * Return no. of bits in a multiprecision integer.
- * @param p		pointer to multiprecision integer
- * @return		no. of bits
- */
-/*@unused@*/ static inline
-unsigned int pgpMpiBits(const byte *p)
-	/*@requires maxRead(p) >= 1 @*/
-	/*@*/
-{
-    return ((p[0] << 8) | p[1]);
-}
-
-/**
- * Return no. of bytes in a multiprecision integer.
- * @param p		pointer to multiprecision integer
- * @return		no. of bytes
- */
-/*@unused@*/ static inline
-unsigned int pgpMpiLen(const byte *p)
-	/*@requires maxRead(p) >= 1 @*/
-	/*@*/
-{
-    return (2 + ((pgpMpiBits(p)+7)>>3));
-}
-	
-/**
- * Convert to hex.
- * @param t		target buffer (returned)
- * @param s		source bytes
- * @param nbytes	no. of bytes
- * @return		target buffer
- */
-/*@unused@*/ static inline
-char * pgpHexCvt(/*@returned@*/ char *t, const byte *s, int nbytes)
-	/*@modifies *t @*/
-{
-    static char hex[] = "0123456789abcdef";
-/*@-boundswrite@*/
-    while (nbytes-- > 0) {
-	unsigned int i;
-	i = *s++;
-	*t++ = hex[ (i >> 4) & 0xf ];
-	*t++ = hex[ (i     ) & 0xf ];
-    }
-    *t = '\0';
-/*@=boundswrite@*/
-    return t;
-}
-
-/**
+/** \ingroup rpmpgp
  * Return hex formatted representation of bytes.
- * @todo Remove static buffer. 
  * @param p		bytes
  * @param plen		no. of bytes
- * @return		hex formatted string
+ * @return		hex formatted string (malloc'ed)
  */
-/*@unused@*/ static inline /*@observer@*/
-char * pgpHexStr(const byte *p, unsigned int plen)
-	/*@*/
-{
-    static char prbuf[8*BUFSIZ];	/* XXX ick */
-    char *t = prbuf;
-    t = pgpHexCvt(t, p, plen);
-    return prbuf;
-}
+char * pgpHexStr(const uint8_t *p, size_t plen);
 
-/**
- * Return hex formatted representation of a multiprecision integer.
- * @todo Remove static buffer. 
- * @param p		bytes
- * @return		hex formatted string
- */
-/*@unused@*/ static inline /*@observer@*/
-const char * pgpMpiStr(const byte *p)
-	/*@requires maxRead(p) >= 3 @*/
-	/*@*/
-{
-    static char prbuf[8*BUFSIZ];	/* XXX ick */
-    char *t = prbuf;
-    sprintf(t, "[%4u]: ", pgpGrab(p, 2));
-    t += strlen(t);
-    t = pgpHexCvt(t, p+2, pgpMpiLen(p)-2);
-    return prbuf;
-}
-
-/**
- * Return string representation of am OpenPGP value.
- * @param vs		table of (string,value) pairs
- * @param val		byte value to lookup
- * @return		string value of byte
- */
-/*@unused@*/ static inline /*@observer@*/
-const char * pgpValStr(pgpValTbl vs, byte val)
-	/*@*/
-{
-    do {
-	if (vs->val == val)
-	    break;
-    } while ((++vs)->val != -1);
-    return vs->str;
-}
-
-/**
- * Return value of an OpenPGP string.
- * @param vs		table of (string,value) pairs
- * @param s		string token to lookup
- * @param se		end-of-string address
- * @return		byte value
- */
-/*@unused@*/ static inline
-int pgpValTok(pgpValTbl vs, const char * s, const char * se)
-	/*@*/
-{
-    do {
-	int vlen = strlen(vs->str);
-	if (vlen <= (se-s) && !strncmp(s, vs->str, vlen))
-	    break;
-    } while ((++vs)->val != -1);
-    return vs->val;
-}
-
-/**
- * Print an OpenPGP value.
- * @param pre		output prefix
- * @param vs		table of (string,value) pairs
- * @param val		byte value to print
- */
-/*@-exportlocal@*/
-void pgpPrtVal(const char * pre, pgpValTbl vs, byte val)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-/*@=exportlocal@*/
-
-/**
- * Print/parse an OpenPGP subtype packet.
- * @param h		packet
- * @param hlen		packet length (no. of bytes)
- * @param sigtype	signature type
- * @return		0 on success
- */
-/*@-exportlocal@*/
-int pgpPrtSubType(const byte *h, unsigned int hlen, pgpSigType sigtype)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-/*@=exportlocal@*/
-
-/**
- * Print/parse an OpenPGP signature packet.
- * @param pp		packet tag/ptr/len
- * @return		0 on success
- */
-/*@-exportlocal@*/
-int pgpPrtSig(const pgpPkt pp)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fileSystem, internalState @*/;
-/*@=exportlocal@*/
-
-/**
- * Print/parse an OpenPGP key packet.
- * @param pp		packet tag/ptr/len
- * @return		0 on success
- */
-int pgpPrtKey(const pgpPkt pp)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fileSystem, internalState @*/;
-
-/**
- * Print/parse an OpenPGP userid packet.
- * @param pp		packet tag/ptr/len
- * @return		0 on success
- */
-/*@-exportlocal@*/
-int pgpPrtUserID(const pgpPkt pp)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fileSystem, internalState @*/;
-/*@=exportlocal@*/
-
-/**
- * Print/parse an OpenPGP comment packet.
- * @param pp		packet tag/ptr/len
- * @return		0 on success
- */
-/*@-exportlocal@*/
-int pgpPrtComment(const pgpPkt pp)
-	/*@globals fileSystem @*/
-	/*@modifies fileSystem @*/;
-/*@=exportlocal@*/
-
-/**
+/** \ingroup rpmpgp
  * Calculate OpenPGP public key fingerprint.
  * @todo V3 non-RSA public keys not implemented.
  * @param pkt		OpenPGP packet (i.e. PGPTAG_PUBLIC_KEY)
  * @param pktlen	OpenPGP packet length (no. of bytes)
- * @retval keyid	publick key fingerprint
+ * @retval keyid	public key fingerprint
  * @return		0 on sucess, else -1
  */
-/*@-exportlocal@*/
-int pgpPubkeyFingerprint(const byte * pkt, unsigned int pktlen,
-		/*@out@*/ byte * keyid)
-	/*@modifies *keyid @*/;
-/*@=exportlocal@*/
+int pgpPubkeyFingerprint(const uint8_t * pkt, size_t pktlen,
+		pgpKeyID_t keyid);
 
-/**
- * Extract OpenPGP public key fingerprint from base64 encoded packet.
- * @todo V3 non-RSA public keys not implemented.
- * @param b64pkt	base64 encoded openpgp packet
- * @retval keyid[8]	public key fingerprint
- * @return		8 (no. of bytes) on success, < 0 on error
- */
-int pgpExtractPubkeyFingerprint(const char * b64pkt, /*@out@*/ byte * keyid)
-	/*@modifies *keyid @*/;
+/** \ingroup rpmpgp
+* Extract OpenPGP public key fingerprint from base64 encoded packet.
+* @todo V3 non-RSA public keys not implemented.
+* @param b64pkt       	base64 encoded openpgp packet
+* @retval keyid		public key fingerprint
+* @return             	8 (no. of bytes) on success, < 0 on error
+*/
+int pgpExtractPubkeyFingerprint(const char * b64pkt, pgpKeyID_t keyid);
 
-/**
- * Return lenth of a OpenPGP packet.
- * @param pkt		OpenPGP packet (i.e. PGPTAG_PUBLIC_KEY)
- * @param pleft		OpenPGP packet length (no. of bytes)
- * @retval pp		packet tag/ptr/len
- * @return		packet length, <0 on error.
- */
-int pgpPktLen(const byte *pkt, unsigned int pleft, /*@out@*/ pgpPkt pp)
-	/*@modifies pp @*/;
-
-/**
- * Print/parse next OpenPGP packet.
- * @param pkt		OpenPGP packet
- * @param pleft		no. bytes remaining
- * @return		-1 on error, otherwise this packet length
- */
-/*@-exportlocal@*/
-int pgpPrtPkt(const byte *pkt, unsigned int pleft)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fileSystem, internalState @*/;
-/*@=exportlocal@*/
-
-/**
+/** \ingroup rpmpgp
  * Print/parse a OpenPGP packet(s).
  * @param pkts		OpenPGP packet(s)
  * @param pktlen	OpenPGP packet(s) length (no. of bytes)
@@ -1292,174 +1021,116 @@ int pgpPrtPkt(const byte *pkt, unsigned int pleft)
  * @param printing	should packets be printed?
  * @return		-1 on error, 0 on success
  */
-int pgpPrtPkts(const byte *pkts, unsigned int pktlen, pgpDig dig, int printing)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies dig, fileSystem, internalState @*/;
+int pgpPrtPkts(const uint8_t *pkts, size_t pktlen, pgpDig dig, int printing);
 
-/**
+/** \ingroup rpmpgp
  * Parse armored OpenPGP packets from a file.
  * @param fn		file name
- * @retval pkt		dearmored OpenPGP packet(s)
+ * @retval pkt		dearmored OpenPGP packet(s) (malloced)
  * @retval pktlen	dearmored OpenPGP packet(s) length in bytes
  * @return		type of armor found
  */
-pgpArmor pgpReadPkts(const char * fn,
-		/*@out@*/ const byte ** pkt, /*@out@*/ size_t * pktlen)
-	/*@globals h_errno, fileSystem, internalState @*/
-	/*@modifies *pkt, *pktlen, fileSystem, internalState @*/;
+pgpArmor pgpReadPkts(const char * fn, uint8_t ** pkt, size_t * pktlen);
 
-/**
+/** \ingroup rpmpgp
+ * Parse armored OpenPGP packets from memory.
+ * @param armor		armored OpenPGP packet string
+ * @retval pkt		dearmored OpenPGP packet(s) (malloced)
+ * @retval pktlen	dearmored OpenPGP packet(s) length in bytes
+ * @return		type of armor found
+ */
+pgpArmor pgpParsePkts(const char *armor, uint8_t ** pkt, size_t * pktlen);
+
+/** \ingroup rpmpgp
  * Wrap a OpenPGP packets in ascii armor for transport.
  * @param atype		type of armor
  * @param s		binary pkt data
  * @param ns		binary pkt data length
  * @return		formatted string
  */
-char * pgpArmorWrap(int atype, const unsigned char * s, size_t ns)
-	/*@*/;
+char * pgpArmorWrap(int atype, const unsigned char * s, size_t ns);
 
-/**
- * Create a container for parsed OpenPGP packates.
+/** \ingroup rpmpgp
+ * Create a container for parsed OpenPGP packet(s).
  * @return		container
  */
-/*@only@*/
-pgpDig pgpNewDig(void)
-	/*@*/;
+pgpDig pgpNewDig(void);
 
-/**
+/** \ingroup rpmpgp
  * Release (malloc'd) data from container.
  * @param dig		container
  */
-void pgpCleanDig(/*@null@*/ pgpDig dig)
-	/*@modifies dig @*/;
+void pgpCleanDig(pgpDig dig);
 
-/**
- * Destroy a container for parsed OpenPGP packates.
+/** \ingroup rpmpgp
+ * Destroy a container for parsed OpenPGP packet(s).
  * @param dig		container
  * @return		NULL always
  */
-/*@only@*/ /*@null@*/
-pgpDig pgpFreeDig(/*@only@*/ /*@null@*/ pgpDig dig)
-	/*@modifies dig @*/;
+pgpDig pgpFreeDig(pgpDig dig);
 
-/**
- * Is buffer at beginning of an OpenPGP packet?
- * @param p		buffer
- * @return		1 if an OpenPGP packet, 0 otherwise
+/** \ingroup rpmpgp
+ * Verify a PGP signature.
+ * @param dig		container
+ * @param hashctx	digest context
+ * @return 		RPMRC_OK on success 
  */
-/*@unused@*/ static inline
-int pgpIsPkt(const byte * p)
-	/*@*/
-{
-/*@-boundsread@*/
-    unsigned int val = *p++;
-/*@=boundsread@*/
-    pgpTag tag;
-    int rc;
+rpmRC pgpVerifySig(pgpDig dig, DIGEST_CTX hashctx);
 
-    /* XXX can't deal with these. */
-    if (!(val & 0x80))
-	return 0;
-
-    if (val & 0x40)
-	tag = (val & 0x3f);
-    else
-	tag = (val >> 2) & 0xf;
-
-    switch (tag) {
-    case PGPTAG_MARKER:
-    case PGPTAG_SYMMETRIC_SESSION_KEY:
-    case PGPTAG_ONEPASS_SIGNATURE:
-    case PGPTAG_PUBLIC_KEY:
-    case PGPTAG_SECRET_KEY:
-    case PGPTAG_PUBLIC_SESSION_KEY:
-    case PGPTAG_SIGNATURE:
-    case PGPTAG_COMMENT:
-    case PGPTAG_COMMENT_OLD:
-    case PGPTAG_LITERAL_DATA:
-    case PGPTAG_COMPRESSED_DATA:
-    case PGPTAG_SYMMETRIC_DATA:
-	rc = 1;
-	break;
-    case PGPTAG_PUBLIC_SUBKEY:
-    case PGPTAG_SECRET_SUBKEY:
-    case PGPTAG_USER_ID:
-    case PGPTAG_RESERVED:
-    case PGPTAG_TRUST:
-    case PGPTAG_PHOTOID:
-    case PGPTAG_ENCRYPTED_MDC:
-    case PGPTAG_MDC:
-    case PGPTAG_PRIVATE_60:
-    case PGPTAG_PRIVATE_62:
-    case PGPTAG_CONTROL:
-    default:
-	rc = 0;
-	break;
-    }
-
-    return rc;
-}
-
-#define CRC24_INIT	0xb704ce
-#define CRC24_POLY	0x1864cfb
-
-/**
- * Return CRC of a buffer.
- * @param octets	bytes
- * @param len		no. of bytes
- * @return		crc of buffer
+/** \ingroup rpmpgp
+ * Return a string identification of a PGP signature/pubkey.
+ * @param digp		signature/pubkey container
+ * @return		string describing the item and parameters
  */
-/*@unused@*/ static inline
-unsigned int pgpCRC(const byte *octets, size_t len)
-	/*@*/
-{
-    unsigned int crc = CRC24_INIT;
-    int i;
+char *pgpIdentItem(pgpDigParams digp);
 
-    while (len--) {
-/*@-boundsread@*/
-	crc ^= (*octets++) << 16;
-/*@=boundsread@*/
-	for (i = 0; i < 8; i++) {
-	    crc <<= 1;
-	    if (crc & 0x1000000)
-		crc ^= CRC24_POLY;
-	}
-    }
-    return crc & 0xffffff;
-}
+/** \ingroup rpmpgp
+ * Perform cryptography initialization.
+ * It must be called before any cryptography can be used within rpm.
+ * It's not normally necessary to call it directly as it's called in
+ * general rpm initialization routines.
+ * @return		0 on success, -1 on failure
+ */
+int rpmInitCrypto(void);
 
-/** \ingroup rpmio
+/** \ingroup rpmpgp
+ * Shutdown cryptography
+ */
+int rpmFreeCrypto(void);
+
+/** \ingroup rpmpgp
  * Duplicate a digest context.
  * @param octx		existing digest context
  * @return		duplicated digest context
  */
-/*@only@*/
-DIGEST_CTX rpmDigestDup(DIGEST_CTX octx)
-	/*@*/;
+DIGEST_CTX rpmDigestDup(DIGEST_CTX octx);
 
-/** \ingroup rpmio
+/** \ingroup rpmpgp
+ * Obtain digest length in bytes.
+ * @param hashalgo	type of digest
+ * @return		digest length, zero on invalid algorithm
+ */
+size_t rpmDigestLength(pgpHashAlgo hashalgo);
+
+/** \ingroup rpmpgp
  * Initialize digest.
  * Set bit count to 0 and buffer to mysterious initialization constants.
  * @param hashalgo	type of digest
  * @param flags		bit(s) to control digest operation
  * @return		digest context
  */
-/*@only@*/ /*@null@*/
-DIGEST_CTX rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
-	/*@*/;
+DIGEST_CTX rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags);
 
-/** \ingroup rpmio
+/** \ingroup rpmpgp
  * Update context with next plain text buffer.
  * @param ctx		digest context
  * @param data		next data buffer
  * @param len		no. bytes of data
  * @return		0 on success
  */
-int rpmDigestUpdate(/*@null@*/ DIGEST_CTX ctx, const void * data, size_t len)
-	/*@modifies ctx @*/;
+int rpmDigestUpdate(DIGEST_CTX ctx, const void * data, size_t len);
 
-/** \ingroup rpmio
+/** \ingroup rpmpgp
  * Return digest and destroy context.
  * Final wrapup - pad to 64-byte boundary with the bit pattern 
  * 1 0* (64-bit count of bits processed, MSB-first)
@@ -1470,14 +1141,65 @@ int rpmDigestUpdate(/*@null@*/ DIGEST_CTX ctx, const void * data, size_t len)
  * @param asAscii	return digest as ascii string?
  * @return		0 on success
  */
-int rpmDigestFinal(/*@only@*/ /*@null@*/ DIGEST_CTX ctx,
-	/*@null@*/ /*@out@*/ void ** datap,
-	/*@null@*/ /*@out@*/ size_t * lenp, int asAscii)
-		/*@modifies *datap, *lenp @*/;
+int rpmDigestFinal(DIGEST_CTX ctx,
+	void ** datap,
+	size_t * lenp, int asAscii);
+
+/** \ingroup rpmpgp
+ * Create a new digest bundle.
+ * @return		New digest bundle
+ */
+rpmDigestBundle rpmDigestBundleNew(void);
+
+/** \ingroup rpmpgp
+ * Free a digest bundle and all contained digest contexts.
+ * @param bundle	digest bundle
+ * @return		NULL always
+ */
+rpmDigestBundle rpmDigestBundleFree(rpmDigestBundle bundle);
+
+/** \ingroup rpmpgp
+ * Add a new type of digest to a bundle.
+ * @param bundle	digest bundle
+ * @param algo		type of digest
+ * @param flags		bit(s) to control digest operation
+ * @return		0 on success
+ */
+int rpmDigestBundleAdd(rpmDigestBundle bundle, pgpHashAlgo algo,
+			rpmDigestFlags flags);
+
+/** \ingroup rpmpgp
+ * Update contexts within bundle with next plain text buffer.
+ * @param bundle	digest bundle
+ * @param data		next data buffer
+ * @param len		no. bytes of data
+ * @return		0 on success
+ */
+int rpmDigestBundleUpdate(rpmDigestBundle bundle, const void *data, size_t len);
+
+/** \ingroup rpmpgp
+ * Return digest from a bundle and destroy context, see rpmDigestFinal().
+ *
+ * @param bundle	digest bundle
+ * @param algo		type of digest to return
+ * @retval datap	address of returned digest
+ * @retval lenp		address of digest length
+ * @param asAscii	return digest as ascii string?
+ * @return		0 on success
+ */
+int rpmDigestBundleFinal(rpmDigestBundle bundle,
+	 pgpHashAlgo algo, void ** datap, size_t * lenp, int asAscii);
+
+/** \ingroup rpmpgp
+ * Duplicate a digest context from a bundle.
+ * @param bundle	digest bundle
+ * @param algo		type of digest to dup
+ * @return		duplicated digest context
+ */
+DIGEST_CTX rpmDigestBundleDupCtx(rpmDigestBundle bundle, pgpHashAlgo algo);
 
 #ifdef __cplusplus
 }
 #endif
-/*@=fcnuse@*/
 
 #endif	/* H_RPMPGP */
