@@ -6,30 +6,40 @@
  *
  */
 
-#include <rpmsw.h>
+#include <rpm/rpmsw.h>
+#include <signal.h>
+#if defined(_RPMSQ_INTERNAL)
+#include <pthread.h>
+#endif
 
-/**
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+/** \ingroup rpmsq
  */
 typedef struct rpmsig_s * rpmsig;
 
-/**
+/** \ingroup rpmsq
  */
 typedef struct rpmsqElem * rpmsq;
 
-/**
+/** \ingroup rpmsq
  * Default signal handler prototype.
  * @param signum	signal number
  * @param info		(siginfo_t) signal info
  * @param context	signal context
  */
-typedef void (*rpmsqAction_t) (int signum, void * info, void * context)
-	/*@*/;
+#ifdef SA_SIGINFO
+typedef void (*rpmsqAction_t) (int signum, void * info, void * context);
+#else
+typedef void (*rpmsqAction_t) (int signum);
+#endif
 
-/*@-redecl@*/
-/*@unchecked@*/
 extern int _rpmsq_debug;
-/*@=redecl@*/
 
+/* XXX make this fully opaque? */
+#if defined(_RPMSQ_INTERNAL)
 /**
  * SIGCHLD queue element.
  */
@@ -43,121 +53,80 @@ struct rpmsqElem {
     rpmtime_t ms_scriptlets;	/*!< Accumulated script duration (msecs). */
     int reaper;			/*!< Register SIGCHLD handler? */
     int pipes[2];		/*!< Parent/child interlock. */
-/*@shared@*/
     void * id;			/*!< Blocking thread id (pthread_t). */
     pthread_mutex_t mutex;	/*!< Signal delivery to thread condvar. */
     pthread_cond_t cond;
 };
+#endif /* _RPMSQ_INTERNAL */
 
-/*@-exportlocal@*/
-/*@unchecked@*/
-extern rpmsq rpmsqQueue;
-/*@=exportlocal@*/
-
-/*@unchecked@*/
-extern sigset_t rpmsqCaught;
-
-#ifdef __cplusplus
-{
-#endif
-
-/**
- * Insert node into from queue.
- * @param elem		node to link
- * @param prev		previous node from queue
- * @return		0 on success
+/** \ingroup rpmsq
+ * Test if given signal has been caught (while signals blocked).
+ * Similar to sigismember() but operates on internal signal queue.
+ * @param signum	signal to test for
+ * @return		1 if caught, 0 if not and -1 on error
  */
-/*@-exportlocal@*/
-int rpmsqInsert(/*@null@*/ void * elem, /*@null@*/ void * prev)
-	/*@globals systemState @*/
-	/*@modifies elem, prev, systemState @*/;
-/*@=exportlocal@*/
+int rpmsqIsCaught(int signum);
 
-/**
- * Remove node from queue.
- * @param elem		node to link
- * @return		0 on success
- */
-/*@-exportlocal@*/
-int rpmsqRemove(/*@null@*/ void * elem)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies elem, fileSystem, internalState @*/;
-/*@=exportlocal@*/
-
-/**
+/** \ingroup rpmsq
  * Default signal handler.
  * @param signum	signal number
  * @param info		(siginfo_t) signal info
  * @param context	signal context
  */
-/*@-exportlocal@*/
-void rpmsqAction(int signum, void * info, void * context)
-	/*@globals rpmsqCaught, rpmsqQueue, errno, fileSystem @*/
-	/*@modifies rpmsqCaught, rpmsqQueue, errno, fileSystem @*/;
-/*@=exportlocal@*/
+#ifdef SA_SIGINFO
+void rpmsqAction(int signum, void * info, void * context);
+#else
+void rpmsqAction(int signum);
+#endif
 
-/**
+/** \ingroup rpmsq
  * Enable or disable a signal handler.
  * @param signum	signal to enable (or disable if negative)
  * @param handler	sa_sigaction handler (or NULL to use rpmsqHandler())
  * @return		no. of refs, -1 on error
  */
-int rpmsqEnable(int signum, /*@null@*/ rpmsqAction_t handler)
-	/*@globals rpmsqCaught, rpmsqQueue, fileSystem, internalState @*/
-	/*@modifies rpmsqCaught, rpmsqQueue, fileSystem, internalState @*/;
+int rpmsqEnable(int signum, rpmsqAction_t handler);
 
-/**
+/** \ingroup rpmsq
  * Fork a child process.
  * @param sq		scriptlet queue element
  * @return		fork(2) pid
  */
-pid_t rpmsqFork(rpmsq sq)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies sq, fileSystem, internalState @*/;
+pid_t rpmsqFork(rpmsq sq);
 
-/**
+/** \ingroup rpmsq
  * Wait for child process to be reaped.
  * @param sq		scriptlet queue element
  * @return		reaped child pid
  */
-pid_t rpmsqWait(rpmsq sq)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies sq, fileSystem, internalState @*/;
+pid_t rpmsqWait(rpmsq sq);
 
-/**
+/** \ingroup rpmsq
  * Call a function in a thread.
  * @param start		function
  * @param arg		function argument
  * @return		thread pointer (NULL on error)
  */
-void * rpmsqThread(void * (*start) (void * arg), void * arg)
-	/*@globals internalState @*/
-	/*@modifies internalState @*/;
+void * rpmsqThread(void * (*start) (void * arg), void * arg);
 
-/**
+/** \ingroup rpmsq
  * Wait for thread to terminate.
  * @param thread	thread
  * @return		0 on success
  */
-int rpmsqJoin(/*@null@*/ void * thread)
-	/*@globals internalState @*/
-	/*@modifies internalState @*/;
+int rpmsqJoin(void * thread);
 
-/**
+/** \ingroup rpmsq
  * Compare thread with current thread.
  * @param thread	thread
  * @return		0 if not equal
  */
-int rpmsqThreadEqual(/*@null@*/ void * thread)
-	/*@globals internalState @*/
-	/*@modifies internalState @*/;
+int rpmsqThreadEqual(void * thread);
 
-/**
+/** \ingroup rpmsq
  * Execute a command, returning its status.
  */
-int rpmsqExecve (const char ** argv)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies fileSystem, internalState @*/;
+int rpmsqExecve (const char ** argv);
 
 #ifdef __cplusplus
 }
