@@ -1,28 +1,25 @@
 #include "system.h"
 const char *__progname;
 
-#include <rpmlib.h>
-#include <rpmmacro.h>
-#include <rpmurl.h>
+#include <popt.h>
 
-#include "rpmdb.h"
-#include "rpmps.h"
-#include "rpmte.h"
-#include "rpmts.h"
+#include <rpm/rpmlib.h>		/* rpmReadConfigFiles */
+#include <rpm/rpmfileutil.h>
+#include <rpm/rpmurl.h>
+#include <rpm/rpmio.h>
+#include <rpm/rpmtag.h>
+#include <rpm/rpmdb.h>
+#include <rpm/rpmps.h>
+#include <rpm/rpmte.h>
+#include <rpm/rpmts.h>
+#include <rpm/rpmds.h>
+#include <rpm/rpmlog.h>
+
+#include "lib/manifest.h"
 
-#include "manifest.h"
-#include "misc.h"
 #include "debug.h"
 
-static int _depends_debug;
-
-static int noAvailable = 1;
-static const char * avdbpath =
-	"/usr/lib/rpmdb/%{_arch}-%{_vendor}-%{_os}/redhat";
-static int noDeps = 0;
-
-static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
-	/*@*/
+static inline const char * identifyDepend(rpmsenseFlags f)
 {
     if (isLegacyPreReq(f))
 	return "PreReq:";
@@ -43,20 +40,20 @@ static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
 }
 
 static int
-do_tsort(const char *fileArgv[])
+do_tsort(const char *fileArgv[], int noDeps)
 {
     rpmts ts = NULL;
-    const char ** pkgURL = NULL;
+    char ** pkgURL = NULL;
     char * pkgState = NULL;
     const char ** fnp;
-    const char * fileURL = NULL;
+    char * fileURL = NULL;
     int numPkgs = 0;
     int numFailed = 0;
     int prevx;
     int pkgx;
-    const char ** argv = NULL;
+    char ** argv = NULL;
     int argc = 0;
-    const char ** av = NULL;
+    ARGV_t av = NULL;
     int ac = 0;
     Header h;
     int rc = 0;
@@ -69,36 +66,11 @@ do_tsort(const char *fileArgv[])
 
     rc = rpmtsOpenDB(ts, O_RDONLY);
     if (rc) {
-	rpmMessage(RPMMESS_ERROR, _("cannot open Packages database\n"));
+	rpmlog(RPMLOG_ERR, "cannot open Packages database\n");
 	rc = -1;
 	goto exit;
     }
 
-#ifdef	DYING
-    /* Load all the available packages. */
-    if (!(noDeps || noAvailable)) {
-	rpmdbMatchIterator mi = NULL;
-	struct rpmdb_s * avdb = NULL;
-	const char * rootdir = "/";
-
-	addMacro(NULL, "_dbpath", NULL, avdbpath, RMIL_CMDLINE);
-	rc = rpmdbOpen(rootdir, &avdb, O_RDONLY, 0644);
-	delMacro(NULL, "_dbpath");
-	if (rc) {
-	    rpmMessage(RPMMESS_ERROR, _("cannot open Available database\n"));
-	    goto endavail;
-	}
-        mi = rpmdbInitIterator(avdb, RPMDBI_PACKAGES, NULL, 0);
-	while ((h = rpmdbNextIterator(mi)) != NULL) {
-	    rpmtsAvailablePackage(ts, h, NULL);
-	}
-
-endavail:
-	if (mi) rpmdbFreeIterator(mi);
-	if (avdb) rpmdbClose(avdb);
-    }
-#endif
-
     /* Build fully globbed list of arguments in argv[argc]. */
     for (fnp = fileArgv; *fnp; fnp++) {
 	av = _free(av);
@@ -145,7 +117,7 @@ restart:
     fileURL = _free(fileURL);
 
     /* Continue processing file arguments, building transaction set. */
-    for (fnp = pkgURL+prevx; *fnp; fnp++, prevx++) {
+    for (fnp = (const char**) pkgURL+prevx; *fnp; fnp++, prevx++) {
 	const char * fileName;
 	FD_t fd;
 
@@ -154,7 +126,7 @@ restart:
 	/* Try to read the header from a package file. */
 	fd = Fopen(*fnp, "r.ufdio");
 	if (fd == NULL || Ferror(fd)) {
-	    rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), *fnp,
+	    rpmlog(RPMLOG_ERR, "open of %s failed: %s\n", *fnp,
 			Fstrerror(fd));
 	    if (fd) Fclose(fd);
 	    numFailed++; *fnp = NULL;
@@ -171,7 +143,7 @@ restart:
 	}
 
 	if (rc != RPMRC_NOTFOUND) {
-	    rpmMessage(RPMMESS_ERROR, _("%s cannot be installed\n"), *fnp);
+	    rpmlog(RPMLOG_ERR, "%s cannot be installed\n", *fnp);
 	    numFailed++; *fnp = NULL;
 	    break;
 	}
@@ -179,7 +151,7 @@ restart:
 	/* Try to read a package manifest. */
 	fd = Fopen(*fnp, "r.fpio");
 	if (fd == NULL || Ferror(fd)) {
-	    rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), *fnp,
+	    rpmlog(RPMLOG_ERR, "open of %s failed: %s\n", *fnp,
 			Fstrerror(fd));
 	    if (fd) Fclose(fd);
 	    numFailed++; *fnp = NULL;
@@ -189,7 +161,7 @@ restart:
 	/* Read list of packages from manifest. */
 	rc = rpmReadPackageManifest(fd, &argc, &argv);
 	if (rc)
-	    rpmError(RPMERR_MANIFEST, _("%s: read manifest failed: %s\n"),
+	    rpmlog(RPMLOG_NOTICE, "%s: read manifest failed: %s\n",
 			fileURL, Fstrerror(fd));
 	Fclose(fd);
 
@@ -212,7 +184,7 @@ restart:
 
 	ps = rpmtsProblems(ts);
 	if (ps) {
-	    rpmMessage(RPMMESS_ERROR, _("Failed dependencies:\n"));
+	    rpmlog(RPMLOG_ERR, "Failed dependencies:\n");
 	    rpmpsPrint(NULL, ps);
 	    ps = rpmpsFree(ps);
 	    rc = -1;
@@ -228,8 +200,6 @@ restart:
     {	rpmtsi pi;
 	rpmte p;
 	rpmte q;
-	unsigned char * selected =
-			alloca(sizeof(*selected) * (rpmtsNElements(ts) + 1));
 	int oType = TR_ADDED;
 
 fprintf(stdout, "digraph XXX {\n");
@@ -268,37 +238,36 @@ exit:
     return rc;
 }
 
-static struct poptOption optionsTable[] = {
- { "noavailable", '\0', 0, &noAvailable, 0,	NULL, NULL},
- { "nodeps", '\0', 0, &noDeps, 0,		NULL, NULL},
- { "verbose", 'v', 0, 0, 'v',			NULL, NULL},
- { NULL,	0, 0, 0, 0,			NULL, NULL}
-};
-
 int
-main(int argc, const char *argv[])
+main(int argc, char *argv[])
 {
     poptContext optCon;
     const char * optArg;
     int arg;
     int ec = 0;
+    int noDeps = 0;
+    struct poptOption optionsTable[] = {
+	{ "nodeps", '\0', 0, &noDeps, 0,		NULL, NULL},
+	{ "verbose", 'v', 0, 0, 'v',			NULL, NULL},
+	{ NULL,	0, 0, 0, 0,			NULL, NULL}
+    };
+
 
 #if HAVE_MCHECK_H && HAVE_MTRACE
     mtrace();	/* Trace malloc only if MALLOC_TRACE=mtrace-output-file. */
 #endif
     setprogname(argv[0]);	/* Retrofit glibc __progname */
+#if defined(ENABLE_NLS)
     (void)setlocale(LC_ALL, "" );
 
-#ifdef  __LCLINT__
-#define LOCALEDIR	"/usr/share/locale"
-#endif
     (void)bindtextdomain(PACKAGE, LOCALEDIR);
     (void)textdomain(PACKAGE);
+#endif
 
-    _depends_debug = 1;
-
-    optCon = poptGetContext("rpmsort", argc, argv, optionsTable, 0);
+    optCon = poptGetContext("rpmsort", argc, (const char **) argv, optionsTable, 0);
+#if RPM_USES_POPTREADDEFAULTCONFIG
     poptReadDefaultConfig(optCon, 1);
+#endif
 
     while ((arg = poptGetNextOpt(optCon)) > 0) {
 	optArg = poptGetOptArg(optCon);
@@ -307,15 +276,15 @@ main(int argc, const char *argv[])
 	    rpmIncreaseVerbosity();
 	    break;
 	default:
-	    fprintf(stderr, _("unknown popt return (%d)"), arg);
+	    fprintf(stderr, "unknown popt return (%d)", arg);
 	    exit(EXIT_FAILURE);
-	    /*@notreached@*/ break;
+	    break;
 	}
     }
 
     rpmReadConfigFiles(NULL, NULL);
 
-    ec = do_tsort(poptGetArgs(optCon));
+    ec = do_tsort(poptGetArgs(optCon), noDeps);
 
     optCon = poptFreeContext(optCon);
 
